--[[ TO DO LIST:
		- Button on TargetFrame to add People to both Teams via dropdown.
]]--

-- Set up AddOn's default values
ArenaLive_Database = {
	["Options"] = {
			Version = "1.0.0",
			Debug = false, -- Debug-Mode on/off
			GameMode = "3v3", -- GameMode (1v1-4v4)
			ClassColorHealthBar = true, -- Class colored Healthbars on/off
			AddonLoaded = false, -- Turns true as soon as the ADDON_LOADED event fires for our AddOn
			maxDebuffs = 20, -- Max Debuffs shown on the unitframes
			isActive = false, -- Turns true as soon as we join the arena
			clickToAdd = false, -- TO DO
			showScoreboard = true; -- Scoreboard shown on/off
			timeLimitMin = 15; -- Scoreboard's TimeLimit Minutes.
			timeLimitSec = 0; -- Scoreboard's TimeLimit Seconds.
			UnitFramesXOffset = 0; -- XOffset of unitframes
			uiScale = 1; -- UIScale modifier
			broadcastCountdown = false; -- Broadcast countdown to party-/raidchat
			castHistoryIconDuration = 7; -- Time in seconds until a castHistoryIcon fades out
		},
	["TeamLeft"] = {
			Name = "",
			Score = 0,
			MemberName1 = "",
			MemberRealm1 = "",
			MemberName2 = "",
			MemberRealm2 = "",
			MemberName3 = "",
			MemberRealm3 = "",
			MemberName4 = "",
			MemberRealm4 = "",
		},
	["TeamRight"] = {
			Name = "",
			Score = 0,
			MemberName1 = "",
			MemberRealm1 = "",
			MemberName2 = "",
			MemberRealm2 = "",
			MemberName3 = "",
			MemberRealm3 = "",
			MemberName4 = "",
			MemberRealm4 = "",
		},
};



--[[ CORE ADDON FUNCTIONS ]]--
--[[	Function: ArenaLive_OnLoad
		Set's up the addon's mainframe, as soon as it is loaded.	
			Variables:
				self: Addon's mainframe (ArenaLiveFrame)
]]--
function ArenaLive_OnLoad(self)

	self.countdown = false;
	self.countdownTime = 0;
	self.nextAlert = 0;
	
	-- Register main frame for Events
	self:RegisterEvent("ADDON_LOADED");
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START");
	
	-- Register Slash-Commands
	SLASH_ARENALIVE1, SLASH_ARENALIVE2 = "/al", "/arenalive";
	

end


--[[	Function: ArenaLive_OnEvent
		Eventhandler for our mainframe	
]]--
function ArenaLive_OnEvent(self, event, ...)
	local addonName = ...;
	local i;
	
	if ( event == "PLAYER_ENTERING_WORLD" ) then
	
		local _, instanceType = IsInInstance()
		
		if ( instanceType ~= "arena" ) then
			ArenaLive_Database.Options.isActive = false;
			
			-- Reset our PartyFrames so that they will be initialised again in the next match
			for i = 1, 4 do
				
				local frame = _G["ArenaLive_LeftTeamFrame"..i];
				
				if (frame.hasPlayerdata) then
					ArenaLiveUnitFrame_UnregisterPlayer(frame)
				end
			end
			
			-- Reset our ArenaFrames so that they will be initialised again in the next match
			for i = 1, 4 do
				local frame = _G["ArenaLive_RightTeamFrame"..i];
				
				if (frame.hasPlayerdata) then
					ArenaLiveUnitFrame_UnregisterPlayer(frame)
				end
			end

			-- Always show BlizzUI outside of the Arena
			PlayerFrame:Hide();
			ArenaLive_ToggleBlizzUI()
			
			-- Reset our ScoreboardTimer as soon as we leave the arena
			ArenaLiveScoarboard_ResetTimer();
			
			-- Reset both Cooldowntrackers
			for i = 1, 30 do
				local cooldownTracker = _G["ArenaLive_CooldowntrackerLeft"];
				if ( cooldownTracker["cd"..i]["enabled"] ) then
					cooldownTracker["cd"..i]["enabled"] = nil;
					cooldownTracker["cd"..i]["unit"] = nil;
					cooldownTracker["cd"..i]["spellID"] = nil;
					cooldownTracker["cd"..i]:SetScript("OnUpdate", nil);
					cooldownTracker["cd"..i]:Hide();
				else
					-- We can break the for-loop here, since there are no active frames left.
					break;
				end
			end

			for i = 1, 30 do
				local cooldownTracker = _G["ArenaLive_CooldowntrackerRight"];
				if ( cooldownTracker["cd"..i]["enabled"] ) then
					cooldownTracker["cd"..i]["enabled"] = nil;
					cooldownTracker["cd"..i]["unit"] = nil;
					cooldownTracker["cd"..i]["spellID"] = nil;
					cooldownTracker["cd"..i]:SetScript("OnUpdate", nil);
					cooldownTracker["cd"..i]:Hide();
				else
					-- We can break the for-loop here, since there are no active frames left.
					break;
				end
			end			
			
			-- Wipe Cooldowntables
			table.wipe(ArenaLive_CooldowntrackerLeft.cooldownList);
			table.wipe(ArenaLive_CooldowntrackerLeft.unitIDs);
			table.wipe(ArenaLive_CooldowntrackerRight.cooldownList);
			table.wipe(ArenaLive_CooldowntrackerRight.unitIDs);
		else
			ArenaLive_Database.Options.isActive = true;
		end
		
	elseif ( event == "ADDON_LOADED" ) and ( addonName == "ArenaLive" ) then

		-- Addon variables are loaded
		ArenaLive_Database.Options.AddonLoaded = true;
		ArenaLive_SystemMessage("AddOn has been loaded successfully! Type \"/al help\" for a list of available commands.", false)
	
		-- Fill our Menu Editboxes with Data
		ArenaLive_SetTeamName(nil, "byLUA", "left", ArenaLive_Database.TeamLeft.Name)
		ArenaLive_SetTeamName(nil, "byLUA", "right", ArenaLive_Database.TeamRight.Name)
		
		for i = 1, 4 do
			ArenaLive_SetPlayer(nil, "byLUA", "name", "TeamLeft", i);
			ArenaLive_SetPlayer(nil, "byLUA", "realm", "TeamLeft", i);
		end
		for i = 1, 4 do		
			ArenaLive_SetPlayer(nil, "byLUA", "name", "TeamRight", i);
			ArenaLive_SetPlayer(nil, "byLUA", "realm", "TeamRight", i);
		end
		
		-- Same for our Dropdown Menu
		local selected = ArenaLive_GetGameModeNumber();
		UIDropDownMenu_SetSelectedID(ArenaLive_OptionsGameModeDropDown, selected, true);
		
		-- Hide player-EditBoxes that are > GameMode number.
		ArenaLive_UpdateEditBoxes()	
		
		-- Toggle our Checkbutton states and set their text
		ArenaLive_OptionsClassColorCheckButtonText:SetText("Class Colors in Healthbars");
		if ( ArenaLive_Database.Options.ClassColorHealthBar ) then
			ArenaLive_OptionsClassColorCheckButton:SetChecked(true);
		else
			ArenaLive_OptionsClassColorCheckButton:SetChecked(nil);
		end

		ArenaLive_OptionsScoreboardCheckButtonText:SetText("Show Scoreboard");
		if ( ArenaLive_Database.Options.showScoreboard ) then
			ArenaLive_OptionsScoreboardCheckButton:SetChecked(true);
			ArenaLive_Scoreboard:Show();
		else
			ArenaLive_OptionsScoreboardCheckButton:SetChecked(nil);
			ArenaLive_Scoreboard:Hide();
		end
		
		ArenaLive_OptionsBroadcastCountdownCheckButtonText:SetText("Broadcast Countdown to Party-/Raidchat");
		if ( ArenaLive_Database.Options.broadcastCountdown ) then
			ArenaLive_OptionsBroadcastCountdownCheckButton:SetChecked(true);
		else
			ArenaLive_OptionsBroadcastCountdownCheckButton:SetChecked(false);
		end
		
		-- Set up TimeLimit Editboxes
		ArenaLive_SetTimeLimit(_G["ArenaLive_OptionsTimeLimitFrameMinEditBox"], "byLUA")
		
		-- Set up our Slider for Unitframe's XOffset
		ArenaLive_OptionsUnitFramesOffsetSliderText:SetText("Unitframe X-Offset");
		ArenaLive_OptionsUnitFramesOffsetSlider:SetValue(ArenaLive_Database.Options.UnitFramesXOffset);
		
		-- Set up our Slider for UIScale
		ArenaLive_OptionsUiScaleSliderText:SetText("UI Scale");
		ArenaLive_OptionsUiScaleSlider:SetValue(ArenaLive_Database.Options.uiScale * 10);
		
		-- Insert Team Names into the Scoreboard-Fontstrings and set Scores
		ArenaLiveScoreboard_SetTeamName(_G["ArenaLive_Scoreboard"], "left");
		ArenaLiveScoreboard_SetTeamName(_G["ArenaLive_Scoreboard"], "right");
		ArenaLiveScoreboard_SetScoreText();
		
		
	elseif ( event == "UNIT_SPELLCAST_CHANNEL_START" ) then
		spellID = select(5,...);
		ArenaLive_KillChanneledSpells(spellID);
	end
end



--[[	Function: ArenaLive_GetGameModeNumber
		GameMode is saved as a string (e.g. 3v3), so we get the first character of this string and make it a number.
]]--
function ArenaLive_GetGameModeNumber()
	-- Extract the first number from the Gamemodestring and set it up as a number. e.g. if GameMode is 3v3 it will return 3
	return tonumber(string.sub(ArenaLive_Database.Options.GameMode, 1, 1)); 
end




--[[	Function: ArenaLive_ToggleBlizzUI
		Shows or Hides most of Blizzard's standard frames
]]--
function ArenaLive_ToggleBlizzUI()
local _, instanceType = IsInInstance()
		
		if (not PlayerFrame:IsShown()) then
			
			-- BuffFrame
			BuffFrame.Show = nil;
			BuffFrame:Show();
			ConsolidatedBuffs.Show = nil;
			if ( GetCVarBool("consolidateBuffs") == true ) then
				ConsolidatedBuffs:Show();
			end
			
			-- GhostFrame
			GhostFrame.Show = nil;
			
			-- CompactRaidFrameManager
			CompactRaidFrameManager.Show = nil;
			CompactRaidFrameManager:Show();
			
			-- Action Bars and Menu
			MultiBarLeft.Show = nil;
			MultiBarLeft:Show();
			MultiBarRight.Show = nil;
			MultiBarRight:Show();			
			MainMenuBar.Show = nil;
			MainMenuBar:Show();

			-- Minimap
			MinimapCluster.Show = nil;
			MinimapCluster:Show();
			
			
			-- Chat
			ChatFrame1.Show = nil;
			ChatFrame1:Show();
			GeneralDockManager.Show = nil;
			GeneralDockManager:Show();
			ChatFrame1ButtonFrame.Show = nil;
			ChatFrame1ButtonFrame:Show();
			FriendsMicroButton.Show = nil;
			FriendsMicroButton:Show();
			ChatFrameMenuButton.Show = nil;
			ChatFrameMenuButton:Show();
			
			-- PlayerFrame
			PlayerFrame.Show = nil;
			PlayerFrame:Show();

			-- TargetFrame
			TargetFrame.Show = nil;
			
			-- FocusFrame
			FocusFrame.Show = nil;
			
			-- Arena Frames are only active in arenas, so we need to check or we'll get a LUA-Error
			
			if (instanceType == "arena") then
				ArenaEnemyFrames.Show = nil;
				ArenaEnemyFrames:Show();
				
				-- Team Frames in Arena
				AlwaysUpFrame1.Show = nil;
				AlwaysUpFrame2.Show = nil;			
				AlwaysUpFrame1:Show();
				AlwaysUpFrame2:Show();
				
			end
		else	
			
			-- BuffFrame
			BuffFrame.Show = function() end;
			BuffFrame:Hide();
			ConsolidatedBuffs.Show = function() end;
			ConsolidatedBuffs:Hide();
			
			-- GhostFrame
			GhostFrame.Show = function() end;
			GhostFrame:Hide();
			
			-- CompactRaidFrameManager
			CompactRaidFrameManager.Show = function() end;
			CompactRaidFrameManager:Hide();
			
			-- Action Bars and Menu
			MultiBarLeft.Show = function() end;
			MultiBarLeft:Hide();
			MultiBarRight.Show = function() end;
			MultiBarRight:Hide();			
			MainMenuBar.Show = function() end;
			MainMenuBar:Hide();
			
			-- Minimap
			MinimapCluster.Show = function() end;
			MinimapCluster:Hide();
			
			-- Chat
			ChatFrame1.Show = function() end;
			ChatFrame1:Hide();
			GeneralDockManager.Show = function() end;
			GeneralDockManager:Hide();
			ChatFrame1ButtonFrame.Show = function() end;
			ChatFrame1ButtonFrame:Hide();
			FriendsMicroButton.Show = function() end;
			FriendsMicroButton:Hide();
			ChatFrameMenuButton.Show = function() end;
			ChatFrameMenuButton:Hide();
			
			-- PlayerFrame
			PlayerFrame.Show = function() end;
			PlayerFrame:Hide();
			
			-- TargetFrame
			TargetFrame.Show = function() end;
			TargetFrame:Hide();
			
			-- FocusFrame
			FocusFrame.Show = function() end;
			FocusFrame:Hide();
			
			-- Arena Frames are only active in arenas, so we need to check or we'll get a LUA-Error
			if (instanceType == "arena") then
				-- Team Frames in Arena
				AlwaysUpFrame1:Hide();
				AlwaysUpFrame1.Show = function() end;
				AlwaysUpFrame2:Hide();
				AlwaysUpFrame2.Show = function() end;
				
				ArenaEnemyFrames.Show = function() end;
				ArenaEnemyFrames:Hide();
			end
		end

end



--[[	Function: ArenaLive_GetUnitID
		Used to get the unitID of a player by comparing a player's name and realm with the arguments of UnitName("unitID");	
			Variables:
				playerName: Name of the player whose unitID we try to get
				realmName: Name of the Realm on which the above-mentioned player is located
				side: Side of the Screen where his Team is shown (left or right)
]]--
function ArenaLive_GetUnitID(playerName, playerRealm, side)

	local thisRealm = GetRealmName();
	local returnUnitID = nil;
	local unit = nil;
	local queryName;
	local queryRealm;
	local isOnSameRealm;
	
	-- First check if Spectator and player are on the same realm, since we don't need the Realmname then.
	if ( playerRealm == thisRealm ) or ( playerRealm == "" ) or (not playerRealm) then
		isOnSameRealm = true;
	else
		isOnSameRealm = nil;
	end
	
	-- Now we need to find out which unitID the player has.
	if (side == "left") then
		
		-- first query for partymembers
		ArenaLive_SystemMessage("Start searching in Party...", true);
		
		for i= 1, 5 do
			queryName, queryRealm = UnitName("party"..i);
			
			if ( isOnSameRealm ) then
				if (playerName == queryName) then
					
					unit = "party"..i;
					break;
				end
			else
				if (playerName == queryName) and (playerRealm == queryRealm) then
					
					unit = "party"..i;
					break;		
				end
			end

		end

		
		-- If the player wasn't in the party, we need to search for him in a Raidenvironment
		if (not unit) then
			
			ArenaLive_SystemMessage("Start searching in Raid...", true);
			
			for i = 1, 40 do
			
				queryName, queryRealm = UnitName("raid"..i);
				if ( isOnSameRealm ) then
					if (playerName == queryName) then
						
						unit = tostring("raid"..i);
						break;
					end
				else
					if (playerName == queryName) and (playerRealm == queryRealm) then
						unit = tostring("raid"..i);

						break;
					end

				end
				
			end

		end
		
		returnUnitID = unit;
		
		if (not returnUnitID) then
			return nil;
		else
			return returnUnitID;
		end
		
	elseif (side == "right") then
		
		-- For the right team we must query the arena1-5 IDs
		for i = 1, 5, 1 do
			
			queryName, queryRealm = UnitName("arena"..i);
			
			if ( isOnSameRealm ) then

				if ( playerName == queryName ) then
					
					unit =  "arena"..i
					break;
				end
				
			else
				
				if ( playerName == queryName ) and ( playerRealm == queryRealm ) then
					unit =  "arena"..i
					break;
				end
				
			end
		end
		
		returnUnitID = unit;
		
		if (not returnUnitID) then
			return nil;
		else
			return returnUnitID;
		end
		
	elseif (side == "target") then
	
		returnUnitID = "target";
		
		return returnUnitID;
	end


end



--[[	Function: ArenaLive_GetClassColors
		Used to get the colorvalues of the different classcolors
			Variables:
				unit: unitID of the player of whom we want to find out the class color.
]]--
function ArenaLive_GetClassColors(unit)

	local classColor = nil;
	local _, unitClass = UnitClass(unit);
	
	if ( unitClass ) then
		classColor = RAID_CLASS_COLORS[unitClass];
	else
		return nil;
	end
	
	return classColor;
end



--[[	Function: ArenaLive_KillChanneledSpells
		Used to track channeled spells, so that they won't flood the CastHistory
			Variables:
				spellID: spellID of the spell that needs to be checked.
]]--
function ArenaLive_KillChanneledSpells(spellID)
	
	name = GetSpellInfo(spellID);
	if ((ArenaLive_SpellsToTrack.ChannelSpells["Names"][name]) and (not ArenaLive_SpellsToTrack.ChannelSpells["SpellIDs"][spellID])) or (not ArenaLive_SpellsToTrack.ChannelSpells["SpellIDs"][spellID]) then
		ArenaLive_SpellsToTrack.ChannelSpells["SpellIDs"][spellID] = name;
		ArenaLive_SystemMessage("New Channeled spells detected: "..name..". Kill it! Kill it with fire!", true);
	end
end



--[[	Function: ArenaLive_SystemMessage
		Handler for the messages our AddOn posts in the default chatframe.
			Variables:
				msg: the message we want to send
				isDebug: true, if it's a debug-message and should only be posted, if the addOn is in debug mode. Else false
]]--
function ArenaLive_SystemMessage(msg, isDebug)
	
	-- This prevents AddOn from spamming DebugMessages, if we don't want it to.
	if ( isDebug == true ) and ( ArenaLive_Database["Options"]["Debug"] == false ) then
		return;	
	end
		
	DEFAULT_CHAT_FRAME:AddMessage("|cFFFF0000ArenaLive:|r "..tostring(msg));

end



--[[	Function: SlashCmdList.ARENALIVE
		Standard Slashcommand function. I won't go into depth with variables here.
]]--
function  SlashCmdList.ARENALIVE(msg, editbox)
	
	if ( msg == "menu" ) then
		ArenaLiveOptions_ToggleVisible();
	elseif ( msg == "countdown" ) then
		ArenaLive_StartCountdown(_G["ArenaLiveFrame"]);
	elseif ( msg == "toggleblizz" ) then
		ArenaLive_ToggleBlizzUI();
	elseif ( msg == "resetscore" ) then
		ArenaLiveScoreboard_ResetTeamScores();
	elseif (msg == "" or msg == "help" ) then
		ArenaLive_SystemMessage("Available slash-commands:", false);
		DEFAULT_CHAT_FRAME:AddMessage("   |cFFFF0000help:|r Show this message.");
		DEFAULT_CHAT_FRAME:AddMessage("   |cFFFF0000menu:|r Show the addon-menu");
		DEFAULT_CHAT_FRAME:AddMessage("   |cFFFF0000countdown:|r Start the countdown and roundtime for a match.");
		DEFAULT_CHAT_FRAME:AddMessage("   |cFFFF0000toggleblizz:|r Show/Hide Blizzard's UI.");
		DEFAULT_CHAT_FRAME:AddMessage("   |cFFFF0000resetscore:|r Reset the scoreboard to 0:0.");
	end
end





--[[ GUI FUNCTIONS ]]--
--[[	Function: ArenaLiveOptions_ToggleVisible
		Shows or Hides the addon's menu.
]]--
function ArenaLiveOptions_ToggleVisible()
	if (ArenaLive_Options:IsShown()) then
		ArenaLive_Options:Hide();
	else
		ArenaLive_Options:Show();
	end
end


--[[	Function: ArenaLive_DropDownOnLoad
		Initialising function for any dropdownmenu of our addon. At the moment there is only one.
]]--
function ArenaLive_DropDownOnLoad(self)
			
			local ArenaLive_GameModes = {
				[1] = {
					["text"] = "1v1",
					["value"] = "1v1",
					["func"] = ArenaLive_SetGameMode,
				},
				[2] = {
					["text"] = "2v2",
					["value"] = "2v2",
					["func"] = ArenaLive_SetGameMode,
				},
				[3] = {
					["text"] = "3v3",
					["value"] = "3v3",
					["func"] = ArenaLive_SetGameMode,
				},
				[4] = {
					["text"] = "4v4",
					["value"] = "4v4",
					["func"] = ArenaLive_SetGameMode,
				},				
			}
			
			for i = 1, 4 do
				
				local info = ArenaLive_GameModes[i];
				UIDropDownMenu_AddButton(info);
			end
			UIDropDownMenu_JustifyText(ArenaLive_OptionsGameModeDropDown, "CENTER")
			-- Add the above information to the options menu as a button.
end



--[[	Function: ArenaLive_SetGameMode
		Sets the GameMode depending on the chosen one in the dropdownmenu
]]--
function ArenaLive_SetGameMode(self)
	
	UIDropDownMenu_SetSelectedID(ArenaLive_OptionsGameModeDropDown, self:GetID(), true);
	ArenaLive_Database.Options.GameMode = self.value

	ArenaLive_SystemMessage("Set game mode: "..ArenaLive_Database.Options.GameMode, false);
	ArenaLive_UpdateEditBoxes()
	
	
end


--[[	Function: ArenaLive_UpdateEditBoxes
		Updates the visible state of our PlayerName and Realm editboxes depending on the chosen GameMode
]]--
function ArenaLive_UpdateEditBoxes()
	
	local gameModeNumber = ArenaLive_GetGameModeNumber();
	
	for i = 1, 4 do
		
		local frame = _G["ArenaLive_OptionsTeamLeftMembersMember"..i];
		
		if (i <= gameModeNumber) then
			frame:Show();
		else
			frame:Hide();
		end
	end

	for i = 1, 4 do
		
		local frame = _G["ArenaLive_OptionsTeamRightMembersMember"..i];
		
		if (i <= gameModeNumber) then
			frame:Show();
		else
			frame:Hide();
		end
	end

end



--[[	Function: ArenaLive_SetTeamName
		Inserts the both teamnames into either the menus editboxes or the scoreboard.
		Variables:
			self: the editbox that changed it's value
			setType: defines the things that will be updated
			side: The teams side on the screen (left or right)
			teamName: the new value of the updated addon parts
			
]]--
function ArenaLive_SetTeamName(self, setType, side, teamName)
		
		if ( setType == "byEditBox" ) then
			
			if ( side == "left" ) then
				
				ArenaLive_Database.TeamLeft.Name = teamName;
				ArenaLiveScoreboard_SetTeamName(_G["ArenaLive_Scoreboard"], "left")
				
			else
			
				ArenaLive_Database.TeamRight.Name = teamName;
				ArenaLiveScoreboard_SetTeamName(_G["ArenaLive_Scoreboard"], "right")
			
			end
			
		elseif ( setType == "byLUA" ) then
		
			if ( side == "left" ) then
				ArenaLive_OptionsTeamLeftNameEditBox:SetText(teamName);
			else
				ArenaLive_OptionsTeamRightNameEditBox:SetText(teamName);
			end
			
		end
end



--[[	Function: ArenaLive_SetPlayer
		Updates Playerinformations
		Variables:
			self: the editbox that changed it's value
			setType: defines the things that will be updated (either the editbox or the database entries)
			whatsToChange: either "name" or "realm", depending on which information needs to be updated
			team: the team's side in a string that fits to the databases structure ("TeamLeft" or "TeamRight")
			memberID: position of the teammember on the screen (1-4)
			
]]--
function ArenaLive_SetPlayer(self, setType, whatsToChange, team, memberID)
		
	local side;
	local unit;
	
	if ( setType == "byEditBox" ) then
	
		if (whatsToChange == "name" ) then
			
			ArenaLive_Database[team]["MemberName"..memberID] = self:GetText();
			
		else
		
			ArenaLive_Database[team]["MemberRealm"..memberID] = self:GetText();
			
		end
		
	elseif ( setType == "byLUA" ) then

		
	
		if (whatsToChange == "name" ) then
			
			local frame = _G["ArenaLive_Options"..team.."MembersMember"..memberID.."NameEditBox"];
			local name = ArenaLive_Database[team]["MemberName"..memberID]
			frame:SetText(name);
			
		else
			local realm = ArenaLive_Database[team]["MemberRealm"..memberID]
			local frame = _G["ArenaLive_Options"..team.."MembersMember"..memberID.."RealmEditBox"];
			frame:SetText(realm);
			
		end
	
	end

	-- Update the affected frame.
	if (team == "TeamRight") then
		side = "right"
	else
		side = "left"
	end

	local unit = ArenaLive_GetUnitID(ArenaLive_Database[team]["MemberName"..memberID], ArenaLive_Database[team]["MemberRealm"..memberID], side);
	
		if (team == "TeamRight" ) then
			local unitFrame = _G["ArenaLive_RightTeamFrame"..memberID];
			
			if ( unitFrame.hasPlayerdata ) then
				ArenaLiveUnitFrame_UnregisterPlayer(unitFrame)
			end
			
			if ( unit ) then
				ArenaLiveUnitFrameRight_OnEvent(unitFrame, "ARENA_OPPONENT_UPDATE", unit, nil)
			end
			
		else
			ArenaLiveUnitFrameLeft_OnEvent(_G["ArenaLive_LeftTeamFrame"..memberID], "UNIT_NAME_UPDATE", unit)
		end
end



--[[	Function: ArenaLive_AddTargetAsPlayer
		Adds the name and realm of a targeted player into the appropriate editboxes
		Variables:
			self: the button that was clicked
			team: the team's side in a string that fits to the databases structure ("TeamLeft" or "TeamRight)
			
]]--
function ArenaLive_AddTargetAsPlayer(self, team)

	local frameID = self:GetParent():GetID();
	local name, realm = UnitName("target");
	
	if (realm == nil) then
		realm = "";
	end
	
	ArenaLive_Database[team]["MemberName"..frameID] = name;
	ArenaLive_Database[team]["MemberRealm"..frameID] = realm;
	
	ArenaLive_SetPlayer(nil, "byLUA", "name", team, frameID)
	ArenaLive_SetPlayer(nil, "byLUA", "realm", team, frameID)
	
	

end



--[[	Function: ArenaLiveOptions_ToggleCheckbutton
		Handler for the state of all checkbuttons in our menu
		Variables:
			self: the button that was clicked
			buttonType: The checkbutton that was clicked
			
]]--
function ArenaLiveOptions_ToggleCheckbutton(self, buttonType)
	
	if ( buttonType == "healthbar" ) then
		
		if ( self:GetChecked()) then
			ArenaLive_Database.Options.ClassColorHealthBar = true;
		else
			ArenaLive_Database.Options.ClassColorHealthBar = false;
		end
	
	elseif ( buttonType == "scoreboard" ) then
	
		if ( self:GetChecked()) then
			ArenaLive_Database.Options.showScoreboard = true;
			ArenaLive_Scoreboard:Show();
		else
			ArenaLive_Database.Options.showScoreboard = false;
			ArenaLive_Scoreboard:Hide();
		end	
	
	elseif ( buttonType == "broadcastCountdown" ) then

		if ( self:GetChecked()) then
			ArenaLive_Database.Options.broadcastCountdown = true;
		else
			ArenaLive_Database.Options.broadcastCountdown = false;
		end		
	
	end
end



--[[	Function: ArenaLive_OnUpdate
		OnUpdate handler for the graphical and broadcasted countdown
			
]]--
function ArenaLive_OnUpdate(self, elapsed)

	if (self.countdown == true) then
		
		self.nextAlert = self.nextAlert + elapsed;
		
		if (self.nextAlert >= 1) then
			
			self.nextAlert = 0;
			self.countdownTime = self.countdownTime - 1;
			if (self.countdownTime > 0) then
				
				if ( ArenaLive_Database.Options.broadcastCountdown == true) then
					if ( IsInRaid() == true ) then
						SendChatMessage("*** GAME STARTS IN "..tostring(self.countdownTime).."***", "BATTLEGROUND");
					elseif ( IsInGroup() == true ) then
						SendChatMessage("*** GAME STARTS IN "..tostring(self.countdownTime).."***", "PARTY");
					end
				end
				
				ArenaLive_CountdownFrameTexture:SetTexture("Interface\\AddOns\\ArenaLive\\textures\\countdown-"..tostring(self.countdownTime));
				ArenaLive_CountdownFrame:Show();
				ArenaLive_CountdownFrameAnimation:Play();
				
			else
				if ( ArenaLive_Database.Options.broadcastCountdown == true) then
					if ( IsInRaid() == true ) then
						SendChatMessage("*** GO! ***", "BATTLEGROUND");	
					elseif ( IsInGroup() == true ) then
						SendChatMessage("*** GO! ***", "PARTY");	
					end
				end
				
				ArenaLive_CountdownFrameTexture:SetTexture("Interface\\AddOns\\ArenaLive\\textures\\countdown-0");
				ArenaLive_CountdownFrame:Show();
				ArenaLive_CountdownFrameAnimation:Play();
			end
		end
		
		if ( self.countdownTime <= 0 ) then
			self:SetScript("OnUpdate", nil);
			self.countdown = false;
			self.countdownTime = 0;
			self.nextAlert = 0;
			ArenaLiveScoarboard_ResetTimer();
			ArenaLiveScoreboard_StartTimer();
			
			local maxPlayers = ArenaLive_GetGameModeNumber();
			
			-- We reset the Casthistory before the actual match begins
			for i = 1, maxPlayers do
				ArenaLiveCasttracker_ReleaseHistoryIcons(_G["ArenaLive_LeftTeamFrame"..i]["castTracker"]);
				ArenaLiveCasttracker_ReleaseHistoryIcons(_G["ArenaLive_RightTeamFrame"..i]["castTracker"]);
			end
		end
	end
end



--[[	Function: ArenaLive_StartCountdown
		Starts the graphical and broadcasted countdown
		Variables:
			mainFrame: The mainframe of the addOn (ArenaLiveFrame)
			
]]--
function ArenaLive_StartCountdown(mainFrame)
	
	if (mainFrame.countdown == false) then
		
		mainFrame.countdown = true;
		mainFrame.countdownTime = 6;
		mainFrame.nextAlert = 1;
		mainFrame:SetScript("OnUpdate", ArenaLive_OnUpdate);
	end
end



--[[	Function: ArenaLiveOptions_SliderOnValueChanged
		OnValueChanged handler for all of our sliders in the menu
		Variables:
			self: The affected slider
			
]]--
function ArenaLiveOptions_SliderOnValueChanged(self)

	local value = self:GetValue() / 10;
	local sliderName = self:GetName();
	
	if ( ArenaLive_Database.Options.AddonLoaded == true ) then
		
		if ( sliderName == "ArenaLive_OptionsUnitFramesOffsetSlider" ) then
	
			local leftTeamFrame = _G["ArenaLive_LeftTeamFrames"];
			local rightTeamFrame = _G["ArenaLive_RightTeamFrames"];
		
			ArenaLive_OptionsUnitFramesOffsetSliderCurrValueText:SetText(tostring(value));
			ArenaLive_Database.Options.UnitFramesXOffset = value;
	
	
			if (leftTeamFrame) then
				-- Changing Offset for PartyFrames
				local point, relativeTo, relativePoint, xOffset, yOffset = leftTeamFrame:GetPoint(1)
				leftTeamFrame:ClearAllPoints();
				xOffset = value;
				leftTeamFrame:SetPoint(point, relativeTo, relativePoint, xOffset, yOffset)
			end
	
			if (rightTeamFrame) then
				-- Changing Offset for ArenaFrames
				local point, relativeTo, relativePoint, xOffset, yOffset = rightTeamFrame:GetPoint(1)
				rightTeamFrame:ClearAllPoints();
				xOffset = 0 - value; -- Negate Number for ArenaFrames
				rightTeamFrame:SetPoint(point, relativeTo, relativePoint, xOffset, yOffset)
			end
			
		elseif ( sliderName == "ArenaLive_OptionsUiScaleSlider" ) then
	
			-- locals of Frames
			local scoreboard = _G["ArenaLive_Scoreboard"];
			local leftTeamFrames = _G["ArenaLive_LeftTeamFrames"];
			local rightTeamFrames = _G["ArenaLive_RightTeamFrames"];
			local leftCooldownTracker = _G["ArenaLive_CooldowntrackerLeft"];
			local rightCooldownTracker = _G["ArenaLive_CooldowntrackerRight"];
			local targetFrame = _G["ArenaLive_TargetFrame"];
			local targetOfTargetFrame = _G["ArenaLive_TargetofTargetFrame"];
	
			-- Set the new Scale
			ArenaLive_Database.Options.uiScale = value;
			ArenaLive_OptionsUiScaleSliderCurrValueText:SetText(tostring(value));
			
			scoreboard:SetScale(value);
			leftTeamFrames:SetScale(value);
			rightTeamFrames:SetScale(value);
			leftCooldownTracker:SetScale(value);
			rightCooldownTracker:SetScale(value);
			targetFrame:SetScale(value);
			targetOfTargetFrame:SetScale(value);
		
		end
	end
	
end



--[[	Function:ArenaLive_SetTimeLimit
		Updates our timelimit depending on the values in either the editboxes or the database
		Variables:
			setType: defines what needs to be changed
			
]]--
function ArenaLive_SetTimeLimit(self, setType)

	local prefix = self:GetParent():GetName();
	local minEditBox = _G[prefix.."MinEditBox"];
	local secEditBox = _G[prefix.."SecEditBox"];

	if ( setType == "byEditBox" ) then

		local minutes = tonumber(minEditBox:GetText());
		local seconds = tonumber(secEditBox:GetText());
	
		-- Now we need to set max values. If someone sets minutes higher than 45 (maximal Arena timelimit) or seconds higher than 59, then we have to change the values accordingly
		if ( minutes > 45 ) then
			minutes = 45;
			minEditBox:SetText(tostring(minutes));
		elseif ( minutes < 10 ) then
			-- Just a small cosmetic change of the text
			minEditBox:SetText(tostring("0"..minutes));
		end
	
		if ( seconds > 59 ) then
			seconds = 59
			secEditBox:SetText(tostring(seconds));
		elseif ( seconds < 10 ) then
			-- Just a small cosmetic change of the text
			secEditBox:SetText(tostring("0"..seconds));
		end
		
		ArenaLive_Database.Options.timeLimitMin = minutes;
		ArenaLive_Database.Options.timeLimitSec = seconds;
		
		
	elseif ( setType == "byLUA" ) then
	
		local minutes = ArenaLive_Database.Options.timeLimitMin;
		local seconds = ArenaLive_Database.Options.timeLimitSec;
		
		-- Just a small cosmetic change of the text
		if ( minutes < 10 ) then
			minutes = "0"..minutes;
		end
		
		if ( seconds < 10 ) then
			seconds = "0"..seconds;
		end
		
		ArenaLive_OptionsTimeLimitFrameMinEditBox:SetText(tostring(minutes));
		ArenaLive_OptionsTimeLimitFrameSecEditBox:SetText(tostring(seconds));	
	
	end	
	
end



--[[ TARGETFRAME FUNCTIONS ]]--
-- Target frame needs to be handled a bit differently than the other frames. So we have to do some copy-paste and edit of some unitframe-functions
function ArenaLiveTargetframe_OnLoad(self, side)
	
	-- Set up some variables and shortcuts for the frame's children
	local prefix = self:GetName();
	self.nameText = _G[prefix.."Name"];
	self.classPortrait = _G[prefix.."ClassPortrait"];
	self.ccOverlay = _G[prefix.."PortraitOverlay"];
	self.healthbar = _G[prefix.."Healthbar"];
	self.powerbar = _G[prefix.."Powerbar"];
	self.castTracker = _G[prefix.."CastTracker"];
	self.buffs = _G[prefix.."Buffs"];
	self.debuffs = _G[prefix.."Debuffs"];
	self.side = side;
	self.elapsed = 0;
	
	ArenaLiveHealthbar_OnLoad(self.healthbar);
	ArenaLivePowerbar_OnLoad(self.powerbar);
	ArenaLiveBuffs_OnLoad(self.buffs);
	ArenaLiveDebuffs_OnLoad(self.debuffs, "target");
	ArenaLiveCasttracker_OnLoad(self.castTracker, side);
	ArenaLiveCcOverlay_OnLoad(self.ccOverlay);
	self.isLoaded = true;	
	
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	self:RegisterEvent("PLAYER_TARGET_CHANGED");
	

	if ( side == "right" ) then
		
		ArenaLiveTargetFrame_InitialisePlayer(self, "target");
		
	else
		self:RegisterEvent("UNIT_TARGET");
		self.debuffs.realUnit = "targettarget";
		
		-- Set up our action for leftclick
		self:RegisterForClicks("LeftButtonUp", "RightButtonUp");
		self:SetAttribute("unit", "targettarget");
		self:SetAttribute("*type1", "target")
		self:SetAttribute("*type2", "focus")
	end	
	
end



function ArenaLiveTargetFrame_InitialisePlayer(self, unit)

	-- if the frame isn't initialised, we can't do anything with it
	if (not self.isLoaded) then
		return;
	end
	
	self.unit = unit;
	self:RegisterEvent("UNIT_DISPLAYPOWER");
	
	ArenaLiveHealthbar_InitialisePlayer(self.healthbar, self.unit);
	ArenaLivePowerbar_InitialisePlayer(self.powerbar, self.unit);
	ArenaLiveDebuffs_InitialisePlayer(self.debuffs, self.unit);
	ArenaLiveBuffs_InitialisePlayer(self.buffs, self.unit);
	ArenaLiveCasttracker_InitialisePlayer(self.castTracker, self.unit);
	ArenaLiveCcOverlay_InitialisePlayer(self.ccOverlay, self.unit);
	self.hasPlayerdata = true;
end



function ArenaLiveTargetFrame_Update(self)
	
	if (UnitExists(self.unit)) and ( ArenaLive_Database.Options.isActive ) then	

		-- Bugfix: Castbar would show up and create a HistoryIcon, if we switched target, while the old target was casting
		self.castTracker.castbar.casting = nil;
		self.castTracker.castbar.channeling = nil;
		self.castTracker.castbar.wasCasting = nil;
		self.castTracker.castbar.fadeOut = nil;
		self.castTracker.castbar:Hide();
		self.castTracker.border:Hide();
		self.castTracker.castbar.effectIcon:Hide();
	
		-- We have to reinitialise the Unit all the time
		ArenaLiveCasttracker_ReleaseHistoryIcons(self.castTracker);
		ArenaLiveTargetFrame_InitialisePlayer(self, "target");
		
		local name = UnitName(self.unit);
		self.nameText:SetText(name);
		
		local _, unitClass = UnitClass(self.unit);
		self.unitClass = unitClass;
		ArenaLive_UpdateClassPortrait(self);
		ArenaLiveHealthbar_Update(self.healthbar, self.unit);
		ArenaLivePowerbar_Update(self.powerbar, self.unit);
		ArenaLiveDebuffs_Refresh(self.debuffs, self.unit);
		ArenaLiveBuffs_Refresh(self.buffs, self.unit);
		
		self:Show();
	else
		-- Reset Healthbar
		self.healthbar.unit = nil;
		self.healthbar:UnregisterEvent("UNIT_HEALTH");
		self.healthbar:UnregisterEvent("UNIT_MAXHEALTH");
		self.healthbar:SetScript("OnEvent", nil);
	
		-- Reset Powerbar
		self.powerbar.unit = nil;
		self.powerbar:UnregisterEvent("UNIT_POWER");
		self.powerbar:UnregisterEvent("UNIT_MAXPOWER");
		self.powerbar:UnregisterEvent("UNIT_DISPLAYPOWER");
		self.powerbar:SetScript("OnEvent", nil);

		-- Reset Debuffhandler
		self.debuffs.unit = nil;
		self.debuffs:UnregisterEvent("UNIT_AURA");
		self.debuffs:SetScript("OnEvent", nil);
	
		-- Reset Casttracker
		self.castTracker.unit = nil;
	
		self.castTracker:UnregisterEvent("UNIT_SPELLCAST_START");
		self.castTracker:UnregisterEvent("UNIT_SPELLCAST_STOP");
		self.castTracker:UnregisterEvent("UNIT_SPELLCAST_FAILED");
		self.castTracker:UnregisterEvent("UNIT_SPELLCAST_INTERRUPTED");
		self.castTracker:UnregisterEvent("UNIT_SPELLCAST_DELAYED");
		self.castTracker:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_START");
		self.castTracker:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE");
		self.castTracker:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_STOP");
		self.castTracker:UnregisterEvent("UNIT_SPELLCAST_INTERRUPTIBLE");
		self.castTracker:UnregisterEvent("UNIT_SPELLCAST_NOT_INTERRUPTIBLE");
		self.castTracker:UnregisterEvent("UNIT_SPELLCAST_SUCCEEDED");
		self.castTracker:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
		self.castTracker:UnregisterEvent("PLAYER_ENTERING_WORLD");	
		self.castTracker:SetScript("OnEvent", nil);
		self.castTracker:SetScript("OnUpdate", nil);	
		
		-- Reset CCOverlay
		self.ccOverlay.unit = nil;	
		self.ccOverlay:SetScript("OnUpdate", nil);
	
		-- We have no playerdata left
		self.hasPlayerdata = nil;		
		
		self:Hide();
	end
end



function ArenaLiveToTFrame_Update(self)
	
	if (UnitExists("targettarget")) and ( ArenaLive_Database.Options.isActive ) then
		
		local totName, totRealm = UnitName("targettarget");
		self:SetScript("OnUpdate", nil);
		-- If we can find something, we just put a real unitID for ToT, so that we can see everything the unit is casting and so on.
		local unit = ArenaLive_GetUnitID(totName, totRealm, "left");
		
		if (not unit) then
			-- If we can't find the target in the raid, we need to search for arena1-5
			unit = ArenaLive_GetUnitID(totName, totRealm, "right");
		end

		if (not unit) then
			ArenaLive_SystemMessage("Sticking with targettarget!", true);
			unit = "targettarget";
			self:SetScript("OnUpdate", ArenaLiveToTFrame_OnUpdate);
		else
			self:SetScript("OnUpdate", nil);
		end
		
		-- Bugfix: Castbar would show up and create a HistoryIcon, if we switched target, while the old target was casting
		self.castTracker.castbar.casting = nil;
		self.castTracker.castbar.channeling = nil;
		self.castTracker.castbar.wasCasting = nil;
		self.castTracker.castbar.fadeOut = nil;
		self.castTracker.castbar:Hide();
		self.castTracker.border:Hide();
		self.castTracker.castbar.effectIcon:Hide();
		
		-- We have to reinitialise the Unit all the time
		ArenaLiveCasttracker_ReleaseHistoryIcons(self.castTracker);
		ArenaLiveTargetFrame_InitialisePlayer(self, unit);
		
		self:Show();
		local name = UnitName(self.unit);
		self.nameText:SetText(name);
		
		local _, unitClass = UnitClass(self.unit);
		self.unitClass = unitClass;
		ArenaLive_UpdateClassPortrait(self);
		ArenaLiveHealthbar_Update(self.healthbar, self.unit);
		ArenaLivePowerbar_Update(self.powerbar, self.unit);
		ArenaLiveDebuffs_Refresh(self.debuffs, self.unit);
		ArenaLiveBuffs_Refresh(self.buffs, self.unit);
		
	else
		-- Reset Healthbar
		self.healthbar.unit = nil;
		self.healthbar:UnregisterEvent("UNIT_HEALTH");
		self.healthbar:UnregisterEvent("UNIT_MAXHEALTH");
		self.healthbar:SetScript("OnEvent", nil);
	
		-- Reset Powerbar
		self.powerbar.unit = nil;
		self.powerbar:UnregisterEvent("UNIT_POWER");
		self.powerbar:UnregisterEvent("UNIT_MAXPOWER");
		self.powerbar:UnregisterEvent("UNIT_DISPLAYPOWER");
		self.powerbar:SetScript("OnEvent", nil);

		-- Reset Debuffhandler
		self.debuffs.unit = nil;
		self.debuffs:UnregisterEvent("UNIT_AURA");
		self.debuffs:SetScript("OnEvent", nil);
	
		-- Reset Casttracker
		self.castTracker.unit = nil;
	
		self.castTracker:UnregisterEvent("UNIT_SPELLCAST_START");
		self.castTracker:UnregisterEvent("UNIT_SPELLCAST_STOP");
		self.castTracker:UnregisterEvent("UNIT_SPELLCAST_FAILED");
		self.castTracker:UnregisterEvent("UNIT_SPELLCAST_INTERRUPTED");
		self.castTracker:UnregisterEvent("UNIT_SPELLCAST_DELAYED");
		self.castTracker:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_START");
		self.castTracker:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE");
		self.castTracker:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_STOP");
		self.castTracker:UnregisterEvent("UNIT_SPELLCAST_INTERRUPTIBLE");
		self.castTracker:UnregisterEvent("UNIT_SPELLCAST_NOT_INTERRUPTIBLE");
		self.castTracker:UnregisterEvent("UNIT_SPELLCAST_SUCCEEDED");
		self.castTracker:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
		self.castTracker:UnregisterEvent("PLAYER_ENTERING_WORLD");
		
		self.castTracker:SetScript("OnEvent", nil);
		self.castTracker:SetScript("OnUpdate", nil);

		-- Reset CCOverlay
		self.ccOverlay.unit = nil;	
		self.ccOverlay:SetScript("OnUpdate", nil);
	
		-- We have no playerdata left
		self.hasPlayerdata = nil;

		self:Hide();
	end
end



function ArenaLiveTargetFrame_OnEvent(self, event, ...)

	if ( event == "PLAYER_ENTERING_WORLD" ) then
		 ArenaLiveTargetFrame_Update(self)
		 
	elseif ( event == "PLAYER_TARGET_CHANGED" ) then
		ArenaLiveTargetFrame_Update(self)
	end	
			
end



function ArenaLiveToTFrame_OnEvent(self, event, ...)

	if ( event == "PLAYER_ENTERING_WORLD" ) then
		 ArenaLiveToTFrame_Update(self);
	elseif ( event == "PLAYER_TARGET_CHANGED" ) then
		ArenaLiveToTFrame_Update(self);
	elseif ( event == "UNIT_TARGET" ) then
		
		local unit = select(1, ...);
		
		if ( unit == "target" ) then
			
			ArenaLiveToTFrame_Update(self);
			
		end
	end	

end



function ArenaLiveToTFrame_OnUpdate(self, elapsed)

	local currHealth = UnitHealth(self.unit); 
	local currPower = UnitPower(self.unit, self.powerType);
	self.elapsed = self.elapsed + elapsed
	
	if ( currHealth ~= self.healthbar.currValue ) then
		ArenaLiveHealthbar_Update(self.healthbar, self.unit);
	end

	if ( currPower ~= self.powerbar.currValue ) then
		ArenaLivePowerbar_Update(self.powerbar, self.unit);
	end
	
	if ( self.elapsed >= 1) then
		self.elapsed = 0;
		ArenaLiveDebuffs_Refresh(self.debuffs, self.unit);
		ArenaLiveBuffs_Refresh(self.buffs, self.unit);
	end
end





--[[ UNITFRAME FUNCTIONS ]]--
-- Quick Note: We divide the loading process of every Unitframe in 2 processes.
-- The first one is handled by the "OnLoad" functions, these set up the frame structure itself.
-- The 2nd set of functions is always called "InitialisePlayer" and fills the frames with player data and registers them for all needed events.

--[[	Function: ArenaLiveUnitFrame_OnLoad
			Variables:
				self: The Frame that is being loaded
				side: Side of the Screen where this frame is shown (left or right)
]]--
function ArenaLiveUnitFrame_OnLoad(self, side)
	
	-- Set up some variables and shortcuts for the frame's children
	local prefix = self:GetName();
	self.nameText = _G[prefix.."Name"];
	self.classPortrait = _G[prefix.."ClassPortrait"];
	self.ccOverlay = _G[prefix.."PortraitOverlay"];
	self.healthbar = _G[prefix.."Healthbar"];
	self.powerbar = _G[prefix.."Powerbar"];
	self.trinket = _G[prefix.."Trinket"];
	self.racial = _G[prefix.."Racial"];
	self.castTracker = _G[prefix.."CastTracker"];
	self.buffs = _G[prefix.."Buffs"];
	self.debuffs = _G[prefix.."Debuffs"];
	self.side = side;
	
	ArenaLiveHealthbar_OnLoad(self.healthbar);
	ArenaLivePowerbar_OnLoad(self.powerbar);
	ArenaLiveCooldown_OnLoad(self.trinket);
	ArenaLiveCooldown_OnLoad(self.racial);
	ArenaLiveBuffs_OnLoad(self.buffs);
	ArenaLiveDebuffs_OnLoad(self.debuffs);
	ArenaLiveCasttracker_OnLoad(self.castTracker, side);
	ArenaLiveCcOverlay_OnLoad(self.ccOverlay);
	self.isLoaded = true;

	-- These are the only events, that are loaded before the frame is filled with playerinformation, because at some points we need them to start the filling-process itself.
	self:RegisterEvent("UNIT_NAME_UPDATE");
	
	-- The 2 Teams need to be registered for different events, since left side is handled by party/raid and right side by arena1-5
	if ( self.side == "left" ) then
		self:RegisterEvent("PARTY_MEMBERS_CHANGED");
		self:RegisterEvent("GROUP_ROSTER_UPDATE");
	elseif ( self.side == "right" ) then
		self:RegisterEvent("ARENA_OPPONENT_UPDATE");
		self:RegisterEvent("ARENA_PREP_OPPONENT_SPECIALIZATIONS"); -- This is a new event from MoP. Need further testing if we can set arena1-5 even if the doors are closed with this.
	end	
end



function ArenaLiveUnitFrame_InitialisePlayer(self, unit)

	-- if the frame isn't initialised, we can't do anything with it
	if (not self.isLoaded) then
		return;
	end

	local _, unitClass = UnitClass(unit);
	local _, unitRace = UnitRace(unit);
	self.unit = unit;
	self.unitClass = unitClass;
	self.unitRace = unitRace;
	
	
	-- Set up our action for leftclick
	self:RegisterForClicks("LeftButtonUp", "RightButtonUp");
	self:SetAttribute("unit", unit);
	self:SetAttribute("*type1", "target")
	self:SetAttribute("*type2", "focus")
	
	self:RegisterEvent("UNIT_DISPLAYPOWER");
	
	ArenaLiveHealthbar_InitialisePlayer(self.healthbar, self.unit);
	ArenaLivePowerbar_InitialisePlayer(self.powerbar, self.unit);
	ArenaLiveCooldown_InitialisePlayer(self.trinket, self.unit, self.unitRace, self.unitClass, "trinket");
	ArenaLiveCooldown_InitialisePlayer(self.racial, self.unit, self.unitRace, self.unitClass, "racial");
	ArenaLiveBuffs_InitialisePlayer(self.buffs, self.unit);
	ArenaLiveDebuffs_InitialisePlayer(self.debuffs, self.unit);
	ArenaLiveCasttracker_InitialisePlayer(self.castTracker, self.unit);
	ArenaLiveCcOverlay_InitialisePlayer(self.ccOverlay, self.unit);
	self.hasPlayerdata = true;
	ArenaLiveUnitFrame_Update(self);
end



function ArenaLiveUnitFrame_UnregisterPlayer(self)
	ArenaLive_SystemMessage("Unregister Player "..tostring(self.unit), false)
	ArenaLiveCasttracker_ReleaseHistoryIcons(self.castTracker)
	-- Reset basic playerdata
	self.unit = nil;
	self.unitClass = nil;
	self.unitRace = nil;

	-- Reset our action for leftclick
	self:SetAttribute("unit", nil);
	self:SetAttribute("*type1", nil)
	
	-- Reset basic events
	self:UnregisterEvent("UNIT_DISPLAYPOWER");
	
	-- Reset Healthbar
	self.healthbar.unit = nil;
	self.healthbar:UnregisterEvent("UNIT_HEALTH");
	self.healthbar:UnregisterEvent("UNIT_MAXHEALTH");
	self.healthbar:SetScript("OnEvent", nil);
	
	-- Reset Powerbar
	self.powerbar.unit = nil;
	self.powerbar:UnregisterEvent("UNIT_POWER");
	self.powerbar:UnregisterEvent("UNIT_MAXPOWER");
	self.powerbar:UnregisterEvent("UNIT_DISPLAYPOWER");
	self.powerbar:SetScript("OnEvent", nil);	

	-- Reset TrinketCD-Timer
	self.trinket.unit = nil;
	self.trinket:UnregisterEvent("UNIT_SPELLCAST_SUCCEEDED");
	self.trinket:SetScript("OnEvent", nil);
	
	-- Reset RacialCD-Timer
	self.racial.unit = nil;
	self.racial:UnregisterEvent("UNIT_SPELLCAST_SUCCEEDED");
	self.racial:SetScript("OnEvent", nil);
	
	-- Reset Debuffhandler
	self.debuffs.unit = nil;
	self.debuffs:UnregisterEvent("UNIT_AURA");
	self.debuffs:SetScript("OnEvent", nil);
	
	-- Reset Casttracker
	self.castTracker.unit = nil;
	
	self.castTracker:UnregisterEvent("UNIT_SPELLCAST_START");
	self.castTracker:UnregisterEvent("UNIT_SPELLCAST_STOP");
	self.castTracker:UnregisterEvent("UNIT_SPELLCAST_FAILED");
	self.castTracker:UnregisterEvent("UNIT_SPELLCAST_INTERRUPTED");
	self.castTracker:UnregisterEvent("UNIT_SPELLCAST_DELAYED");
	self.castTracker:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_START");
	self.castTracker:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE");
	self.castTracker:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_STOP");
	self.castTracker:UnregisterEvent("UNIT_SPELLCAST_INTERRUPTIBLE");
	self.castTracker:UnregisterEvent("UNIT_SPELLCAST_NOT_INTERRUPTIBLE");
	self.castTracker:UnregisterEvent("UNIT_SPELLCAST_SUCCEEDED");
	self.castTracker:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
	self.castTracker:UnregisterEvent("PLAYER_ENTERING_WORLD");
	
	self.castTracker:SetScript("OnEvent", nil);
	self.castTracker:SetScript("OnUpdate", nil);

	-- Reset CCOverlay
	self.ccOverlay.unit = nil;	
	self.ccOverlay:SetScript("OnUpdate", nil);
	
	-- We have no playerdata left
	self.hasPlayerdata = nil;
	
	self:Hide();
end



function ArenaLiveUnitFrameLeft_OnEvent(self, event, ...)
	
	if (not self.isLoaded) or (ArenaLive_Database.Options.AddonLoaded ~= true) then
		ArenaLive_SystemMessage("Frame wasn't initialised or SavedVariables weren't loaded.", true)
		return;
	end

	local frameID = self:GetID();
	local maxPlayers = ArenaLive_GetGameModeNumber();
	local eventUnit = ...;
	
	if ( ArenaLive_Database.Options.isActive ) and (frameID <= maxPlayers) then
			
			local frameName = self:GetName();
			local playerName = ArenaLive_Database.TeamLeft["MemberName"..frameID];
			local playerRealm = ArenaLive_Database.TeamLeft["MemberRealm"..frameID];
		
		
		if (playerName ~= "" and (not self.hasPlayerdata) and (event == "UNIT_NAME_UPDATE" or event == "PARTY_MEMBERS_CHANGED" or event == "GROUP_ROSTER_UPDATE")) then
			
			-- Filling the Frame with Userdata and initialising it
			local unit = ArenaLive_GetUnitID(playerName, playerRealm, "left");
			
			
			if (unit) then			
				ArenaLiveUnitFrame_InitialisePlayer(self, unit);
				ArenaLiveCooldownTracker_UpdateUnitID(_G["ArenaLive_CooldowntrackerLeft"], unit, nil, "add");
			else
				ArenaLive_SystemMessage("Couldn't find fitting unitID for Frame \""..frameName.."\"!", true);
			end
			
		elseif (self.hasPlayerdata) and ( event == "UNIT_NAME_UPDATE" or "PARTY_MEMBERS_CHANGED" or "GROUP_ROSTER_UPDATE") then
			
			-- Since the Groupsetup can be changed, we always need to be aware of changes to the unitID of the players we wan't to track.
			local name, realm = UnitName(self.unit);
			
			-- We need to equalise the realmstrings
			if (realm == nil) then
				realm = "";
			end
			
			if (playerName ~= name) or ((playerName == name) and (playerRealm ~= realm)) then
				
				-- The current unitID is no longer correct, so we need to find a new one.
				ArenaLive_SystemMessage("Searching for new unitID...", true);
				local newUnitID = nil;
				newUnitID = ArenaLive_GetUnitID(playerName, playerRealm, self.side);
				
				
				ArenaLive_SystemMessage("newUnitID: "..tostring(newUnitID), true);
				
				if (not newUnitID) then
					-- We couldn't find a new unitID, this means that the player has left the group.
					ArenaLiveCooldownTracker_UpdateUnitID(_G["ArenaLive_CooldowntrackerLeft"], self.unit, nil, "delete");
					ArenaLiveUnitFrame_UnregisterPlayer(self)
				else
					
					-- We must do this before the new unitID gehts initialised, since we need the old unitID of this frame to remove it from the Cooldowntrackerlist
					ArenaLiveCasttracker_ReleaseHistoryIcons(self.castTracker)
					ArenaLiveCooldownTracker_UpdateUnitID(_G["ArenaLive_CooldowntrackerLeft"], newUnitID, self.unit, "modify");
					ArenaLiveUnitFrame_InitialisePlayer(self, newUnitID)
				end	
				
			else
				ArenaLive_SystemMessage("unitID for \""..playerName.."\" and \""..playerRealm.."\" is still the same. (name = "..tostring(name)..", realm = "..tostring(realm)..")", true);				
			end
		end
	else
	
		if (self.hasPlayerdata) then
			ArenaLiveUnitFrame_UnregisterPlayer(self);
			self:Hide();
		end
	end
end



function ArenaLiveUnitFrameRight_OnEvent(self, event, ...)
	--[[	seen = An enemy gets visible
			unseen = An enemy starts stealth/invis or releases his/her Ghost
			destroyed = An enemy leaves the Arena
			cleared = This fires, after you leave the Arena ]]--

	if (not self.isLoaded) or (ArenaLive_Database.Options.AddonLoaded ~= true) then
		ArenaLive_SystemMessage("Frame wasn't initialised or SavedVariables weren't loaded.", true)
		return;
	end
	
	local frameID = self:GetID();
	local maxPlayers = ArenaLive_GetGameModeNumber();
	
	if ( ArenaLive_Database.Options.isActive ) and (frameID <= maxPlayers) then
			
		local arg1, arg2 = ...;		
		local frameName = self:GetName();		
		local playerName = ArenaLive_Database.TeamRight["MemberName"..frameID];
		local playerRealm = ArenaLive_Database.TeamRight["MemberRealm"..frameID];
		
		-- Filling Arenaframes with playerdata.
		if (playerName ~= "" ) and (not self.hasPlayerdata) and (event == "ARENA_OPPONENT_UPDATE" or event == "UNIT_NAME_UPDATE") then
			
			local unit = ArenaLive_GetUnitID(playerName, playerRealm, "right");
			
			if (unit) then
					
				ArenaLiveUnitFrame_InitialisePlayer(self, unit)
				ArenaLiveCooldownTracker_UpdateUnitID(_G["ArenaLive_CooldowntrackerRight"], unit, nil, "add");
			else
				ArenaLive_SystemMessage("Couldn't find fitting unitID for ArenaFrame"..frameID, false);
			end		
			
		elseif (self.hasPlayerdata) and (event == "ARENA_OPPONENT_UPDATE") and (arg1 == self.unit) then
			
			-- In contrast to our partyframes, we only need to fill them once, but we have to keep track about the states seen, unseen, destroyed and cleared.
			if (arg2 == "seen" or arg2 == "destroyed") then
				
				self.healthbar.lockColor = false;
				self.powerbar.lockColor = false;
				
				ArenaLiveUnitFrame_Update(self);

			elseif (arg2 == "unseen") then
				
				self.healthbar:SetStatusBarColor(0.5, 0.5, 0.5);
				self.healthbar.lockColor = true;
				self.powerbar:SetStatusBarColor(0.5, 0.5, 0.5);
				self.powerbar.lockColor = true;
				
			elseif (arg2 == "cleared") then
				self.healthbar.lockColor = false;
				self.powerbar.lockColor = false;
				
				ArenaLiveCooldownTracker_UpdateUnitID(_G["ArenaLive_CooldowntrackerLeft"], self.unit, nil, "delete");
				ArenaLiveUnitFrame_UnregisterPlayer(self);
			end
		end
	
	else
		self:Hide();
	end
	
end



function ArenaLiveUnitFrame_Update(self)
	
	if (not self.hasPlayerdata) then
		return;
	end
	
	if ( UnitGUID(self.unit)) then
		self:Show();	
		local name = UnitName(self.unit)
		self.nameText:SetText(name);
		ArenaLive_UpdateClassPortrait(self);
		ArenaLiveHealthbar_Update(self.healthbar, self.unit);
		ArenaLivePowerbar_Update(self.powerbar, self.unit);
		ArenaLiveDebuffs_Refresh(self.debuffs, self.unit);
		ArenaLiveBuffs_Refresh(self.buffs, self.unit)
	end
end



function ArenaLive_UpdateClassPortrait(self)
	
	if (not self.hasPlayerdata) then
		return;
	end
	
	local isPlayer = UnitIsPlayer(self.unit);
	
	-- Sometimes the class needs to be updated, so we check if our class is still the same.
	local _, classFileName  = UnitClass(self.unit);
		
	if (classFileName == nil) then
		_, classFileName = GetPlayerInfoByGUID(UnitGUID(self.unit));
	end	
	
	-- New Class name so we have to update it.
	if (classFileName) and ( classFileName ~= self.unitClass ) then
		ArenaLive_SystemMessage("New Class: classFileName: "..tostring(classFileName).." and self.unitClass: "..tostring(self.unitClass), true);
		self.unitClass = classFileName;
	end
	
	if ( self.classPortrait ) then
		local texture = "Interface\\Icons\\INV_Misc_QuestionMark";
		
		if (self.unitClass) and (isPlayer) then
			
			
			texture = "Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes";
			self.classPortrait:SetTexture(texture);
			self.classPortrait:SetTexCoord(unpack(CLASS_ICON_TCOORDS[self.unitClass]));
		elseif (self.unitClass) and (not isPlayer) then
			-- Pets and NPCs get their portrait as texture!
			SetPortraitTexture(self.classPortrait, self.unit);
			self.classPortrait:SetTexCoord(0.1, 0.9, 0, 0.9);
		else
			self.classPortrait:SetTexture(texture);
			self.classPortrait:SetTexCoord(0, 1, 0, 1);
		end
		
	end
	
end





--[[ STATUSBAR FUNCTIONS ]]--
function ArenaLiveHealthbar_OnLoad(self)
	
	local prefix = self:GetName();
	self.text = _G[prefix.."Text"];
	self.background = _G[prefix.."Background"];
	self.zeroText = DEAD;
	self.lockColor = false;
	self.disconnected = false;
end



function ArenaLiveHealthbar_InitialisePlayer(self, unit)

	self.unit = unit;
	self:RegisterEvent("UNIT_HEALTH");
	self:RegisterEvent("UNIT_MAXHEALTH");
	self:SetScript("OnEvent", ArenaLiveHealthbar_OnEvent);
end



function ArenaLiveHealthbar_OnEvent(self, event, ...)
	
	if ( event == "UNIT_HEALTH" ) or ( event == "UNIT_MAXHEALTH" ) then	
		
		if (UnitGUID(self.unit)) then
			ArenaLiveHealthbar_Update(self, ...);
		end
	end
end

function ArenaLiveHealthbar_Update(self, unit)
	
	if (not self) then
		return;
	end
	
	local color = nil;
	local bgColor = nil;
	local isPlayer = UnitIsPlayer(self.unit);
	
	if ( self.unit == unit ) then
		local maxValue = UnitHealthMax(unit);
		local currValue = UnitHealth(unit);
		
		-- Safety check used by Blizzard's Unitframes to prevent empty bars
		if ( maxValue == 0 ) then
			maxValue = 1;
		end
		
		
		self:SetMinMaxValues(0, maxValue);
		
		self.disconnected = not UnitIsConnected(unit);
		if (self.disconnected) then
			if (not self.lockColor) then
				self.background:SetTexture(0.3, 0.3, 0.3, 1);
			end
			self:SetValue(maxValue);
			self.currValue = maxValue;
		else
			self:SetValue(currValue);
			self.currValue = currValue;
			
			if (not self.lockColor) then
				if ( ArenaLive_Database.Options.ClassColorHealthBar ) and ( isPlayer ) then
					
					color = ArenaLive_GetClassColors(unit);
					
					if (color.r) then
						self:SetStatusBarColor(color.r, color.g, color.b);
						self.background:SetTexture(color.r*0.1, color.g*0.1, color.b*0.1, 1);
					else
						-- fall back to green, if we can't get the class color
						self:SetStatusBarColor(0.0, 1.0, 0.0);
						self.background:SetTexture(0, 0.1, 0, 1);
					end
					
				else
					self:SetStatusBarColor(0.0, 1.0, 0.0);
					self.background:SetTexture(0, 0.1, 0, 1);
				end
			end
		end
		
		-- Set the Background of the Healthbar to gray, if the unit is dead
		if (currValue == 0) then
			self.background:SetTexture(0.2, 0.2, 0.2, 1);
			--self.isZero = 1;
		end

		ArenaLiveStatusbarText_Update(self);
	end
end





function ArenaLivePowerbar_OnLoad(self)
	
	local prefix = self:GetName();
	self.text = _G[prefix.."Text"];
	self.background = _G[prefix.."Background"];
	self.zeroText = "0%";
end



function ArenaLivePowerbar_InitialisePlayer(self, unit)

	self.unit = unit;
	self:RegisterEvent("UNIT_POWER");
	self:RegisterEvent("UNIT_MAXPOWER");
	self:RegisterEvent("UNIT_DISPLAYPOWER");
	self:SetScript("OnEvent", ArenaLivePowerbar_OnEvent);
end



function ArenaLivePowerbar_OnEvent(self, event, ...)
	
	if ( event == "UNIT_POWER" ) or( event == "UNIT_DISPLAYPOWER") or ( event == "UNIT_MAXPOWER" ) then
		ArenaLivePowerbar_Update(self, ...);
	end
end



function ArenaLivePowerbar_Update(self, unit)
	if (not self) then
		return;
	end
	
	if (unit == self.unit) then
		-- First update the powerType
		ArenaLivePowerbar_UpdateType(self);
		
		local maxValue = UnitPowerMax(unit, self.powerType);
		local currValue = UnitPower(unit, self.powerType);
		
		self:SetMinMaxValues(0, maxValue);
		self:SetValue(currValue);
		self.currValue = currValue;
		ArenaLiveStatusbarText_Update(self)
	end
end



function ArenaLivePowerbar_UpdateType(self)
	
	if (not self) or (not self.unit) then
		return;
	end
	
	local powerType, powerToken, altR, altG, altB = UnitPowerType(self.unit);
	local colorInfo = PowerBarColor[powerToken];
	
	if (colorInfo) then
		self:SetStatusBarColor(colorInfo.r, colorInfo.g, colorInfo.b);
		local r = colorInfo.r * 0.1;
		local g = colorInfo.g * 0.1;
		local b = colorInfo.b * 0.1;
		self.background:SetTexture(r, g, b, 1);
	else
		if (not altR) then
			-- couldn't find a power token entry...default to indexing by power type or just mana if we don't have that either. Copied from Blizz' code
			colorInfo = PowerBarColor[powerType] or PowerBarColor["MANA"];
		else
			local r = altR * 0.1;
			local g = altG * 0.1;
			local b = altB * 0.1;
			self.background:SetTexture(r, g, b, 1);			
			self:SetStatusBarColor(altR, altG, altB);
		end
	end
	
	self.powerType = powerType;

end





function ArenaLiveStatusbarText_Update(self)
	if ( self.text ) then
		local value = self:GetValue();
		local minValue, maxValue = self:GetMinMaxValues();
		
		if ( value == 0 ) and (self.zeroText) then
			self.text:SetText(self.zeroText);
		else
			value = tostring(math.ceil((value/maxValue) * 100)).."%";
			self.text:SetText(value);
		end
		
		
	end
end





--[[ CASTTRACKER FUNCTIONS ]]--
function ArenaLiveCasttracker_OnLoad(self, side)
	
	local prefix = self:GetName();
	self.border = _G[prefix.."Border"];
		self.border.animation = _G[prefix.."BorderAnimation"];
	self.vertexColorSuccess = { ["r"] = 0, ["g"] = 1, ["b"] = 0, ["a"] = 1,};
	self.vertexColorFailed = { ["r"] = 1, ["g"] = 0, ["b"] = 0, ["a"] = 1,};
	self.side = side
	
	self.historyIcon1 = _G[prefix.."HistoryIcon1"];
		self.historyIcon1.texture = _G[prefix.."HistoryIcon1Texture"];
		self.historyIcon1.border = _G[prefix.."HistoryIcon1Border"];
		self.historyIcon1.animation = _G[prefix.."HistoryIcon1Animation"];
		self.historyIcon1.animationLast = _G[prefix.."HistoryIcon1AnimationLast"];
		self.historyIcon1.animationFadeIn = _G[prefix.."HistoryIcon1AnimationFadeIn"];
		self.historyIcon1.animationFadeOut = _G[prefix.."HistoryIcon1AnimationFadeOut"];
		self.historyIcon1.vertexOverlay = _G[prefix.."HistoryIcon1VertexOverlay"];
		self.historyIcon1.interrupted = _G[prefix.."HistoryIcon1Interrupted"];
		self.historyIcon1.wasMoved = 0;
		self.historyIcon1.duration = 0;
		self.historyIcon1.unitClass = nil;
		
	self.historyIcon2 = _G[prefix.."HistoryIcon2"];
		self.historyIcon2.texture = _G[prefix.."HistoryIcon2Texture"];
		self.historyIcon2.border = _G[prefix.."HistoryIcon2Border"];
		self.historyIcon2.animation = _G[prefix.."HistoryIcon2Animation"];
		self.historyIcon2.animationLast = _G[prefix.."HistoryIcon2AnimationLast"];
		self.historyIcon2.animationFadeIn = _G[prefix.."HistoryIcon2AnimationFadeIn"];
		self.historyIcon2.animationFadeOut = _G[prefix.."HistoryIcon2AnimationFadeOut"];
		self.historyIcon2.vertexOverlay = _G[prefix.."HistoryIcon2VertexOverlay"];
		self.historyIcon2.interrupted = _G[prefix.."HistoryIcon2Interrupted"];
		self.historyIcon2.wasMoved = 0;
		self.historyIcon2.duration = 0;
		self.historyIcon2.unitClass = nil;

		
	self.historyIcon3 = _G[prefix.."HistoryIcon3"];
		self.historyIcon3.texture = _G[prefix.."HistoryIcon3Texture"];
		self.historyIcon3.border = _G[prefix.."HistoryIcon3Border"];
		self.historyIcon3.animation = _G[prefix.."HistoryIcon3Animation"];
		self.historyIcon3.animationLast = _G[prefix.."HistoryIcon3AnimationLast"];
		self.historyIcon3.animationFadeIn = _G[prefix.."HistoryIcon3AnimationFadeIn"];
		self.historyIcon3.animationFadeOut = _G[prefix.."HistoryIcon3AnimationFadeOut"];
		self.historyIcon3.vertexOverlay = _G[prefix.."HistoryIcon3VertexOverlay"];
		self.historyIcon3.interrupted = _G[prefix.."HistoryIcon3Interrupted"];
		self.historyIcon3.wasMoved = 0;
		self.historyIcon3.duration = 0;
		self.historyIcon3.unitClass = nil;

		
	self.historyIcon4 = _G[prefix.."HistoryIcon4"];
		self.historyIcon4.texture = _G[prefix.."HistoryIcon4Texture"];
		self.historyIcon4.border = _G[prefix.."HistoryIcon4Border"];
		self.historyIcon4.animation = _G[prefix.."HistoryIcon4Animation"];
		self.historyIcon4.animationLast = _G[prefix.."HistoryIcon4AnimationLast"];
		self.historyIcon4.animationFadeIn = _G[prefix.."HistoryIcon4AnimationFadeIn"];
		self.historyIcon4.animationFadeOut = _G[prefix.."HistoryIcon4AnimationFadeOut"];
		self.historyIcon4.vertexOverlay = _G[prefix.."HistoryIcon4VertexOverlay"];
		self.historyIcon4.interrupted = _G[prefix.."HistoryIcon4Interrupted"];
		self.historyIcon4.wasMoved = 0;
		self.historyIcon4.duration = 0;
		self.historyIcon4.unitClass = nil;

	self.historyIcon5 = _G[prefix.."HistoryIcon5"];
		self.historyIcon5.texture = _G[prefix.."HistoryIcon5Texture"];
		self.historyIcon5.border = _G[prefix.."HistoryIcon5Border"];
		self.historyIcon5.animation = _G[prefix.."HistoryIcon5Animation"];
		self.historyIcon5.animationLast = _G[prefix.."HistoryIcon5AnimationLast"];
		self.historyIcon5.animationFadeIn = _G[prefix.."HistoryIcon5AnimationFadeIn"];
		self.historyIcon5.animationFadeOut = _G[prefix.."HistoryIcon5AnimationFadeOut"];
		self.historyIcon5.vertexOverlay = _G[prefix.."HistoryIcon5VertexOverlay"];
		self.historyIcon5.interrupted = _G[prefix.."HistoryIcon5Interrupted"];
		self.historyIcon5.wasMoved = 0;
		self.historyIcon5.duration = 0;
		self.historyIcon5.unitClass = nil;


	self.historyIcon6 = _G[prefix.."HistoryIcon6"];
		self.historyIcon6.texture = _G[prefix.."HistoryIcon6Texture"];
		self.historyIcon6.border = _G[prefix.."HistoryIcon6Border"];
		self.historyIcon6.animation = _G[prefix.."HistoryIcon6Animation"];
		self.historyIcon6.animationLast = _G[prefix.."HistoryIcon6AnimationLast"];
		self.historyIcon6.animationFadeIn = _G[prefix.."HistoryIcon6AnimationFadeIn"];
		self.historyIcon6.animationFadeOut = _G[prefix.."HistoryIcon6AnimationFadeOut"];
		self.historyIcon6.vertexOverlay = _G[prefix.."HistoryIcon6VertexOverlay"];
		self.historyIcon6.interrupted = _G[prefix.."HistoryIcon6Interrupted"];
		self.historyIcon6.wasMoved = 0;
		self.historyIcon6.duration = 0;
		self.historyIcon6.unitClass = nil;

	-- We need two more history icons than we are actually showing, as a sort of backup
	self.historyIcon7 = _G[prefix.."HistoryIcon7"];
		self.historyIcon7.texture = _G[prefix.."HistoryIcon7Texture"];
		self.historyIcon7.border = _G[prefix.."HistoryIcon7Border"];
		self.historyIcon7.animation = _G[prefix.."HistoryIcon7Animation"];
		self.historyIcon7.animationLast = _G[prefix.."HistoryIcon7AnimationLast"];
		self.historyIcon7.animationFadeIn = _G[prefix.."HistoryIcon7AnimationFadeIn"];
		self.historyIcon7.animationFadeOut = _G[prefix.."HistoryIcon7AnimationFadeOut"];
		self.historyIcon7.vertexOverlay = _G[prefix.."HistoryIcon7VertexOverlay"];
		self.historyIcon7.interrupted = _G[prefix.."HistoryIcon7Interrupted"];	
		self.historyIcon7.wasMoved = 0;
		self.historyIcon7.duration = 0;
		self.historyIcon7.unitClass = nil;
		
	self.historyIcon8 = _G[prefix.."HistoryIcon8"];
		self.historyIcon8.texture = _G[prefix.."HistoryIcon8Texture"];
		self.historyIcon8.border = _G[prefix.."HistoryIcon8Border"];
		self.historyIcon8.animation = _G[prefix.."HistoryIcon8Animation"];
		self.historyIcon8.animationLast = _G[prefix.."HistoryIcon8AnimationLast"];
		self.historyIcon8.animationFadeIn = _G[prefix.."HistoryIcon8AnimationFadeIn"];
		self.historyIcon8.animationFadeOut = _G[prefix.."HistoryIcon8AnimationFadeOut"];
		self.historyIcon8.vertexOverlay = _G[prefix.."HistoryIcon8VertexOverlay"];
		self.historyIcon8.interrupted = _G[prefix.."HistoryIcon8Interrupted"];	
		self.historyIcon8.wasMoved = 0;
		self.historyIcon8.duration = 0;
		self.historyIcon8.unitClass = nil;
		
	self.historyIcon9 = _G[prefix.."HistoryIcon9"];
		self.historyIcon9.texture = _G[prefix.."HistoryIcon9Texture"];
		self.historyIcon9.border = _G[prefix.."HistoryIcon9Border"];
		self.historyIcon9.animation = _G[prefix.."HistoryIcon9Animation"];
		self.historyIcon9.animationLast = _G[prefix.."HistoryIcon9AnimationLast"];
		self.historyIcon9.animationFadeIn = _G[prefix.."HistoryIcon9AnimationFadeIn"];
		self.historyIcon9.animationFadeOut = _G[prefix.."HistoryIcon9AnimationFadeOut"];
		self.historyIcon9.vertexOverlay = _G[prefix.."HistoryIcon9VertexOverlay"];
		self.historyIcon9.interrupted = _G[prefix.."HistoryIcon9Interrupted"];	
		self.historyIcon9.wasMoved = 0;
		self.historyIcon9.duration = 0;
		self.historyIcon9.unitClass = nil;
		
	self.castbar = _G[prefix.."Castbar"];
		self.castbar.text = _G[prefix.."CastbarText"];
		self.castbar.animation = _G[prefix.."CastbarAnimation"];
		self.castbar.icon = _G[prefix.."CastbarIcon"];
			self.castbar.icon.texture = _G[prefix.."CastbarIconTexture"];
			self.castbar.icon.interrupted = _G[prefix.."CastbarIconInterrupted"];
			self.castbar.icon.animation = _G[prefix.."CastbarIconAnimation"];
		self.castbar.effectIcon = _G[prefix.."CastbarEffectIcon"];
			self.castbar.effectIcon.texture = _G[prefix.."CastbarEffectIconTexture"];
			self.castbar.effectIcon.interrupted = _G[prefix.."CastbarEffectIconInterrupted"];
			self.castbar.effectIcon.animation = _G[prefix.."CastbarEffectIconAnimation"];
			
		self.castbar.casting = nil;
		self.castbar.channeling = nil;
		self.castbar.showCastbar = true;
		self.usedIcon = nil;
		self.currentUnitClass = nil;
		self.castbar.castInterrupted = nil;
		self.castbar.castFailed = nil;
		self.castbar.wasCasting = nil;
end



function ArenaLiveCasttracker_InitialisePlayer(self, unit)
	
	self.unit = unit;
	
	self:RegisterEvent("UNIT_SPELLCAST_START");
	self:RegisterEvent("UNIT_SPELLCAST_STOP");
	self:RegisterEvent("UNIT_SPELLCAST_FAILED");
	self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED");
	self:RegisterEvent("UNIT_SPELLCAST_DELAYED");
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START");
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE");
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP");
	self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTIBLE");
	self:RegisterEvent("UNIT_SPELLCAST_NOT_INTERRUPTIBLE");
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED");
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	
	self:SetScript("OnEvent", ArenaLiveCasttracker_OnEvent);
	self:SetScript("OnUpdate", ArenaLiveCasttracker_OnUpdate);
end



function ArenaLiveCasttracker_OnEvent(self, event, ...)

	
	if (event ~= "COMBAT_LOG_EVENT_UNFILTERED") then
		local unit = ...;

		if (unit ~= self.unit) then
			return;
		end
		
		if (event == "PLAYER_ENTERING_WORLD" ) then

			local nameSpell = UnitCastingInfo(unit);
			local nameChannel = UnitChannelInfo(unit);
			
			if (nameChannel) then
				event = "UNIT_SPELLCAST_CHANNEL_START";			
			elseif (nameSpell) then
				event = "UNIT_SPELLCAST_START";				
			else
				ArenaLiveCasttracker_FinishSpell(self);
			end
			
		elseif ( event == "UNIT_SPELLCAST_START" ) then
			ArenaLiveCasttracker_CastStart(self, ...);
		elseif ( event == "UNIT_SPELLCAST_STOP" or event == "UNIT_SPELLCAST_CHANNEL_STOP") then
			ArenaLiveCasttracker_CastOrChannelStop(self, event, ...)
		elseif ( event == "UNIT_SPELLCAST_FAILED" or event == "UNIT_SPELLCAST_INTERRUPTED" ) then
			ArenaLiveCasttracker_CastFailedOrInterrupted(self, event, ...)
		elseif ( event == "UNIT_SPELLCAST_DELAYED" ) then
			ArenaLiveCasttracker_CastDelay(self, unit);
		elseif ( event == "UNIT_SPELLCAST_CHANNEL_START" ) then
			ArenaLiveCasttracker_ChannelStart(self, ...);
		elseif ( event == "UNIT_SPELLCAST_CHANNEL_UPDATE" ) then
			ArenaLiveCasttracker_ChannelUpdate(self, unit)
		elseif ( event == "UNIT_SPELLCAST_INTERRUPTIBLE" ) then
			-- TO DO: SHIELD IMAGE
		elseif ( event == "UNIT_SPELLCAST_NOT_INTERRUPTIBLE" ) then
			-- TO DO: SHIELD IMAGE
		elseif ( event == "UNIT_SPELLCAST_SUCCEEDED" ) then
			ArenaLiveCasttracker_CastSucceeded(self, ...);
		end
	else
		ArenaLiveCastertracker_CombatlogHandler(self, event, ...)
	end
	
end



function ArenaLiveCasttracker_CastStart(self, ...)
	
	local unit = select(1, ...);
	local spellID = select(5, ...);
	local name, subText, text, texture, startTime, endTime, isTradeSkill, lineID, notInterruptible = UnitCastingInfo(unit);
	self.castbar.currSpellID = spellID;
	
	if (not name) then
		self.castbar:Hide();
		return;
	end
	
	-- First we must stop animations that are in progress, if there are any
	if (self.castbar.animation:IsPlaying()) then
		self.castbar.animation:Stop();
		self.border.animation:Stop();
	end
	
	self.castbar:SetStatusBarColor(1.0, 0.7, 0.0);
	
	-- We need to divide startTime and endTime by 1000, since UnitCastingInfo() returns the times in milliseconds
	self.castbar.value = (GetTime() - (startTime / 1000));
	self.castbar.maxValue = (endTime - startTime) / 1000;
	self.castbar:SetMinMaxValues(0, self.castbar.maxValue);
	self.castbar:SetValue(self.castbar.value);
	
	self.castbar.text:SetText(text);
	
	self.castbar.icon.texture:SetTexture(texture);
	
	self.castbar.casting = 1;
	self.castbar.lineID = lineID;
	self.castbar.channeling = nil;
	self.castbar.fadeOut = nil;
	self.castbar.castFailed = nil;
	self.castbar.castInterrupted = nil;
	self.castbar.wasCasting = 1;
	
	if ( self.castbar.showCastbar ) then
		self.castbar:Show();
		self.border:Show();
		self.castbar.icon:Show();
	end

end



function ArenaLiveCasttracker_CastOrChannelStop(self, event, ...)
	
	local lineID = select(4,...);
	
	-- lineID always returns 0 for channeled spells. So we don't need to check it, when channeled spells are stopped.
	if ( (self.castbar.casting) and (event == "UNIT_SPELLCAST_STOP" ) and (lineID == self.castbar.lineID) ) or ( (self.castbar.channeling) and (event == "UNIT_SPELLCAST_CHANNEL_STOP") ) then
		
		self.castbar:SetValue(self.castbar.maxValue);
		
		if ( event == "UNIT_SPELLCAST_STOP" ) then
		
				if (self.castbar.value <= self.castbar.maxValue - 0.1) then
					self.castbar.casting = nil;
					self.castbar.castFailed = 1;
					self.castbar.fadeOut = 1;
				else
					self.castbar:SetStatusBarColor(0.0, 1.0, 0.0);
					self.castbar.casting = nil;
					self.castbar.fadeOut = 1;
				end
		else
					self.castbar:SetStatusBarColor(0.0, 1.0, 0.0);
					self.castbar.channeling = nil;
					self.castbar.fadeOut = 1;
		end
	end
end




function ArenaLiveCasttracker_CastFailedOrInterrupted(self, event, ...)
	
	local lineID = select(4,...);

	if (self.castbar:IsShown()) and (self.castbar.casting and self.castbar.lineID == lineID) and (not self.castbar.fadeOut) then
		self.castbar:SetValue(self.castbar.maxValue);
		self.castbar:SetStatusBarColor(1.0, 0.0, 0.0);
		
		self.castbar.text:SetText(FAILED);
		
		local spellID = select(5, ...);
		local name, _, icon, _, _, _, castingTime, _, _ = GetSpellInfo(spellID);		
		
		self.castbar.castFailed = 1;
		self.castbar.casting = nil;
		self.castbar.channeling = nil;
		self.castbar.fadeOut = 1;
	end
end



function ArenaLiveCasttracker_CastDelay(self, unit)

	if ( self.castbar:IsShown() ) then
		
		local name, subText, text, texture, startTime, endTime, isTradeSkill, lineID, notInterruptible = UnitCastingInfo(unit);
		
		if (not name) then
			return;
		end
		
		-- We need to divide startTime and endTime by 1000, since UnitCastingInfo() returns the times in milliseconds
		self.castbar.value = (GetTime() - (startTime / 1000));
		self.castbar.maxValue = (endTime - startTime) / 1000;
		self.castbar:SetMinMaxValues(0, self.castbar.maxValue);

		if ( not self.castbar.casting ) then
			self.castbar:SetStatusBarColor(1.0, 0.7, 0.0);
		end
		
		self.castbar.casting = 1;
		self.castbar.wasCasting = 1;
		self.castbar.channeling = nil;
		self.castbar.fadeOut = nil;
		self.castbar.castFailed = nil;
		self.castbar.castInterrupted = nil;
	end
end



function ArenaLiveCasttracker_ChannelStart(self, ...)
	
	local unit = select(1, ...);
	local spellID = select(5, ...);
	local name, subText, text, texture, startTime, endTime, isTradeSkill, notInterruptible = UnitChannelInfo(unit)
	self.castbar.currSpellID = spellID;
	
	if (not ArenaLive_SpellsToTrack.ChannelSpells["SpellIDs"][spellID]) then
		ArenaLive_KillChanneledSpells(spellID);
	end
	
	if (not name) then
		self.castbar:Hide();
		return;
	end

	-- If the Animation of Border+Castbar is still in progress, we need to stop it, so that they can be used for the next cast.
	if (self.castbar.animation:IsPlaying()) then
		self.castbar.animation:Stop();
		self.border.animation:Stop();
	end

	self.castbar:SetStatusBarColor(0.0, 1.0, 0.0);	
	
	-- We need to divide startTime and endTime by 1000, since UnitCastingInfo() returns the times in milliseconds
	self.castbar.value = ((endTime / 1000) - GetTime());
	self.castbar.maxValue = (endTime - startTime) / 1000;
	self.castbar:SetMinMaxValues(0, self.castbar.maxValue);
	self.castbar:SetValue(self.castbar.value);
	self.castbar.text:SetText(text);	
	
	-- The "effectIcon" is an exact copy of castbar.icon and is used for dealing with animations later on.
	self.castbar.icon.texture:SetTexture(texture);
	

	self.castbar.casting = nil;
	self.castbar.channeling = 1;
	self.castbar.fadeOut = nil;
	self.castbar.castFailed = nil;
	self.castbar.castInterrupted = nil;
	self.castbar.wasCasting = 1;

	
	if ( self.castbar.showCastbar ) then
		self.castbar:Show();
		self.border:Show();
		self.castbar.icon:Show();
	end
	
end



function ArenaLiveCasttracker_ChannelUpdate(self, unit)
	if ( self.castbar:IsShown()) then
		local name, subText, text, texture, startTime, endTime, isTradeSkill, lineID, notInterruptible = UnitChannelInfo(unit);

		if (not name) then
			self.castbar:Hide();
			return;
		end	
		
		-- We need to divide startTime and endTime by 1000, since UnitCastingInfo() returns the times in milliseconds
		self.castbar.value = ((endTime / 1000) - GetTime());
		self.castbar.maxValue = (endTime - startTime) / 1000;
		self.castbar:SetMinMaxValues(0, self.castbar.maxValue);
		self.castbar:SetValue(self.castbar.value);
	end
end



function ArenaLiveCasttracker_FinishSpell(self)
	self.castbar:SetStatusBarColor(0.0, 1.0, 0.0);
	self.castbar.fadeOut = 1;
	self.castbar.casting = nil;
	self.castbar.channeling = nil;
end



function ArenaLiveCasttracker_CastSucceeded(self, ...)
	local unit = ...;
	local spellID = select(5, ...);
	local name, _, icon, _, _, _, castingTime, _, _ = GetSpellInfo(spellID);	
	
	if (unit == self.unit ) then

		-- Track them nasty Channelticks!
		if (ArenaLive_SpellsToTrack.ChannelSpells.Names[name]) then
			ArenaLive_KillChanneledSpells(spellID)
		end	
	
		-- We need to dinstinguish between instant casts, channeled and casted casts. But if there is no icon, we won't do anything.
		
		if (castingTime == 0) and (not ArenaLive_SpellsToTrack.ChannelSpells["SpellIDs"][spellID]) and ( not self.castbar.fadOut) then
			ArenaLiveCasttracker_HistoryHandler(self, spellID, true, false, "instant")
		end
		
	end
end



function ArenaLiveCastertracker_CombatlogHandler(self, event, ...)

	local event2 = select(2, ...);
	local sourceName = select(5, ...);
	local selfName = GetUnitName(self.unit, true);
	local destName = select(9, ...);
	local destGUID = select(8, ...);
	local destClass = nil;
	local isPlayer = UnitIsPlayer(destName)
	
	if (event2 == "SPELL_HEAL" or event2 == "SPELL_DAMAGE" or event2 == "SPELL_CAST_SUCCESS" ) then

		if (sourceName == selfName) then
			if (destName) and (isPlayer) then
				_, destClass = UnitClass(destName);
				
				-- If we can't find it by Name, we try using GUID
				if (not destClass) then
					_, destClass = GetPlayerInfoByGUID(destGUID)
				end
			end
			self.currentUnitClass = destClass;
		end	
	elseif (event2 == "SPELL_INTERRUPT") then			
		
		if (destName == selfName) and (self.castbar:IsShown()) then
			self.castbar.castInterrupted = true;
			self.currentUnitClass = nil;
		end
	end
end



function ArenaLiveCasttracker_OnUpdate(self, elapsed)
	
	if (self.castbar.casting) then
		self.castbar.value = self.castbar.value + elapsed;
		
		if (self.castbar.value >= self.castbar.maxValue) then
			self.castbar:SetValue(self.castbar.maxValue);
			ArenaLiveCasttracker_FinishSpell(self);
		end
		
		self.castbar:SetValue(self.castbar.value);
		
	elseif (self.castbar.channeling) then
		
		self.castbar.value = self.castbar.value - elapsed;
		
		if ( self.castbar.value <= 0 ) then
			self.castbar:SetValue(self.castbar.maxValue);
			ArenaLiveCasttracker_FinishSpell(self);
			return;
		end
		
		self.castbar:SetValue(self.castbar.value);
		
	elseif (self.castbar.fadeOut) then
		self.castbar.fadeOut = nil;
		if (self.castbar.effectIcon.animation:IsPlaying()) then
			
			-- If the icon is already in use, we have to stop it's animation to start a new one.
			-- This may not look pretty, but it is needed to prevent displayerrors. 
			self.castbar.effectIcon.animation:Stop();
			self.castbar.effectIcon:Hide();
			
		end

		if (self.castbar.castInterrupted == true) then
			self.castbar.castInterrupted = false;
			self.usedIcon = ArenaLiveCasttracker_HistoryHandler(self, self.castbar.currSpellID, nil, true, "cast")
			self.castbar:SetStatusBarColor(1.0, 0.0, 0.0);
			self.castbar.text:SetText(INTERRUPTED);
			self.castbar.effectIcon.interrupted:Show();	
		elseif ( self.castbar.castFailed == 1 ) then
			self.castbar:SetValue(self.castbar.maxValue);
			self.castbar:SetStatusBarColor(1.0, 0.0, 0.0);
			self.castbar.text:SetText(FAILED);		
		else
			self.usedIcon = ArenaLiveCasttracker_HistoryHandler(self, self.castbar.currSpellID, true, false, "cast")			
		end
		
		self.castbar.wasCasting = nil;
		self.castbar.animation:Play();
		self.border.animation:Play();
	end
end



function ArenaLiveCasttracker_HistoryHandler(self, spellID, wasSuccessful, wasInterrupted, castType)

	local name, rank, icon, powerCost, isFunnel, powerType, castingTime, minRange, maxRange = GetSpellInfo(spellID);
	local iconFound = 0;
	local returnIcon;

	for i = 1, 9 do
		
		frame = self["historyIcon"..i];
		
		if (frame.texture:GetTexture()) then
			
			frame:Show(); -- Bugfix. Sometimes the Icons disappear.
			-- This Frame is already active and shown or will be shown soon, so we just move it to create space for our new one.
			frame.wasMoved = frame.wasMoved + 1;
			
			if ( not frame.animationFadeOut:IsPlaying() ) then
				if ( frame.wasMoved > 5 ) then
					frame.animationLast:Play();
				else
					frame.animation:Play();
				end
			end
		else
			
			if (iconFound == 0) then
				-- This is our new frame, so we fill it with data and get it rolling.
				frame.texture:SetTexture(icon);

				local borderColor;
				
				if ( wasSuccessful ) then
					borderColor = self.vertexColorSuccess;
					frame.vertexOverlay:SetVertexColor(1, 1, 1, 1);
					frame.vertexOverlay:Hide();
					self.castbar.effectIcon.texture:SetVertexColor(1, 1, 1, 1);
				else
					borderColor = self.vertexColorFailed;
					frame.vertexOverlay:Show();
					frame.vertexOverlay:SetVertexColor(borderColor.r, borderColor.g, borderColor.b, borderColor.a);
					self.castbar.effectIcon.texture:SetVertexColor(borderColor.r, borderColor.g, borderColor.b, borderColor.a);
				end

				if ( wasInterrupted ) then
					frame.interrupted:Show();
				else
					frame.interrupted:Hide();
				end
				
				if ( castType == "instant" ) then
					frame:SetAlpha(0);
					frame:Show();
					frame.animationFadeIn:Play();
					
				elseif (castType == "cast" ) then
					self.castbar.effectIcon.texture:SetTexture(icon);
					self.castbar.effectIcon:Show();
					self.castbar.effectIcon.animation:Play();
					
				end
				
				-- Set duration for our OnUpdate-ScriptHandler of the HistoryIcon, so we can fade it out after 10 Seconds.
				frame.duration = ArenaLive_Database.Options.castHistoryIconDuration;
				returnIcon = i;
				iconFound = 1;
			end
			
		end
	end
	return returnIcon;

end



function ArenaLiveCasttracker_HistoryIconReset(historyIcon)
	
	historyIcon:SetScript("OnUpdate", nil);
	historyIcon.texture:SetTexture();
	historyIcon.border:SetVertexColor();
	historyIcon.interrupted:Hide();
	historyIcon.wasMoved = 0;
	historyIcon.duration = 0;
	local point, relativeTo, relativePoint, xOffset, yOffset = historyIcon:GetPoint(1);
	historyIcon:ClearAllPoints();
	
	if (point == "TOPLEFT") then
		xOffset = 4;
	else
		xOffset = -4;
	end
	historyIcon:SetPoint(point , relativeTo, relativePoint, xOffset, yOffset);
end


-- TO DO HISTORY ICON FADE
function ArenaLiveCasttracker_HistoryIconOnUpdate(self, elapsed)

	-- First we check, if the Button is currently active
	if ( self.texture:GetTexture() ) then 
	
		self.duration = self.duration - elapsed;
		
		if ( self.duration <= 0 ) then
			-- Time's up, so we fade the button out.
			self:SetScript("OnUpdate", nil);
			
			if ( not self.animationLast:IsPlaying()  ) then
				self.animationFadeOut:Play();
			end
		end
	end
end



function ArenaLiveCasttracker_ReleaseHistoryIcons(self)

	if (not self) then
		return;
	end

	ArenaLive_SystemMessage("Releasing all History Icons for: "..tostring(self:GetName()), true);
	
	for i = 1, 9 do
		self["historyIcon"..i]:Hide();
		self["historyIcon"..i]:SetScript("OnUpdate", nil);
		self["historyIcon"..i]["texture"]:SetTexture();
		self["historyIcon"..i]["border"]:SetVertexColor();
		self["historyIcon"..i]["interrupted"]:Hide();
		self["historyIcon"..i]["wasMoved"] = 0;
		self["historyIcon"..i]["duration"] = 0;
		local point, relativeTo, relativePoint, xOffset, yOffset = self["historyIcon"..i]:GetPoint(1);
		self["historyIcon"..i]:ClearAllPoints();
		if (point == "TOPLEFT") then
			xOffset = 4;
		else
			xOffset = -4;
		end
		
		self["historyIcon"..i]:SetPoint(point , relativeTo, relativePoint, xOffset, yOffset);
	end
end


function ArenaLiveCasttracker_AnimationHandler(self, scriptType, animationType)
	
	local frame = self:GetParent();
	local castTracker = frame:GetParent();
	
	if (animationType == "effectIcon") then

		if( castTracker.usedIcon ) then
			
			n = castTracker.usedIcon;
			local unitClass = castTracker.currentUnitClass;
			self.currentUnitClass = nil;
			
			if (castTracker["historyIcon"..n].texture:GetTexture()) then
				if (unitClass) then
					local color = RAID_CLASS_COLORS[unitClass];
					castTracker["historyIcon"..n]["border"]:SetVertexColor(color.r, color.g, color.b, 1)
					castTracker["historyIcon"..n]["border"]:Show();
				else
					castTracker["historyIcon"..n]["border"]:Hide();
				end

			
				castTracker["historyIcon"..n]:Show();
				-- Start OnUpdate-EventHandler for fadeOut-animation
				castTracker["historyIcon"..n]:SetScript("OnUpdate", ArenaLiveCasttracker_HistoryIconOnUpdate);
			end
		end
		
		frame:Hide();
		frame.interrupted:Hide();
	elseif (animationType == "historyIcon") then
	
		n = frame.wasMoved;
		frame:Show(); -- Bugfix. Sometimes the Icons disappear.
		local point, relativeTo, relativePoint, xOffset, yOffset = frame:GetPoint(1);
		frame:ClearAllPoints();
		if (castTracker.side == "left") then
			xOffset = (30 * n) + 4;
		else
			xOffset = (-30 * n) - 4;
		end
		frame:SetPoint(point, relativeTo, relativePoint, xOffset, yOffset);

		
	elseif (animationType == "historyIconFadeIn") then
	
		local unitClass = castTracker.currentUnitClass;
	
		if (unitClass) then
			local color = RAID_CLASS_COLORS[unitClass];
			frame["border"]:SetVertexColor(color.r, color.g, color.b, 1)
			frame["border"]:Show();
		else
			frame["border"]:Hide();
		end
	
		frame:SetAlpha(1);
		self.currentUnitClass = nil;
		frame:SetScript("OnUpdate", ArenaLiveCasttracker_HistoryIconOnUpdate);
		
	elseif (animationType == "historyIconFadeOut") then
	
		frame:Hide();
		ArenaLiveCasttracker_HistoryIconReset(frame);
			
	elseif (animationType == "castbar") then
		
		frame:Hide();
		
	end
end



--[[ BUFF FUNCTIONS ]]--
function ArenaLiveBuffs_OnLoad(self, unit)
	
	local prefix = self:GetName();
	local maxDebuffs;

	
	-- We create shortcuts to each of the small debuff frames
	for i = 1, 10, 1 do
		self["buff"..i] = _G[prefix.."Buff"..i];
		self["buffBorder"..i] = _G[prefix.."Buff"..i.."Border"];
		self["buffIcon"..i] = _G[prefix.."Buff"..i.."Icon"];
		self["buffCooldown"..i] = _G[prefix.."Buff"..i.."Cooldown"];
		self["buffStacks"..i] = _G[prefix.."Buff"..i.."Stacks"];
		self["buff"..i]["spellID"] = nil;
		self["buff"..i]["expiration"] = nil;		
	end
end



function ArenaLiveBuffs_InitialisePlayer(self, unit)
	
	self.unit = unit;
	self:RegisterEvent("UNIT_AURA");
	self:SetScript("OnEvent", ArenaLiveBuffs_OnEvent);
end



function ArenaLiveBuffs_OnEvent(self, event, ...)
	
	local unit = ...;
	
	if ( event == "UNIT_AURA" ) and ( unit == self.unit) then
		ArenaLiveBuffs_Refresh(self, unit)
	end
end



function ArenaLiveBuffs_Refresh(self, unit)
	
	local maxBuffs = 40;
	local totalBuffFrames = 10;
	local i = 1;
	local buffFrame = 1;
	local parent = self:GetParent();
	
	-- Thanks J. for this awesome while-loop idea! :>
	while i <= maxBuffs and buffFrame <= totalBuffFrames do
		
		local name, _, icon, count, _, duration, expirationTime, _, _, _, spellID = UnitBuff(unit, i);
		
		if (name) then
			
			-- We've found a spell, so we fill the current frame with it.
			if (ArenaLive_SpellsToTrack.ShownBuffs[spellID]) then
			
				self["buffIcon"..buffFrame]:SetTexture(icon);
				self["buffBorder"..buffFrame]:SetVertexColor(0, 1, 0);
				self["buffCooldown"..buffFrame]:SetCooldown(expirationTime - duration, duration); -- Note for myself: expirationtime - complete duration gives us the starttime of the debuff. This way the debuff will alwyays be shown with the right cooldownprogress.
				self["buff"..buffFrame]["spellID"] = spellID;
				
				if (count > 1 ) then
					self["buffStacks"..buffFrame]:SetText(count);
				else
					self["buffStacks"..buffFrame]:SetText("");
				end
				
				self["buff"..buffFrame]["expiration"] = expirationTime;
				self["buff"..buffFrame]:Show();				
				
				buffFrame = buffFrame + 1;
				
			end
			
			i = i + 1;
		else
			break;
		end
	end
	
	while buffFrame <= totalBuffFrames do
		self["buff"..buffFrame]:Hide();
		buffFrame = buffFrame + 1;
	end
	
end



--[[ DEBUFFHANDLER FUNCTIONS ]]--
function ArenaLiveDebuffs_OnLoad(self, unit)
	
	local prefix = self:GetName();
	local maxDebuffs;
	if (unit == "target" or unit == "targettarget" ) then
		maxDebuffs = 10;
	else
		maxDebuffs = ArenaLive_Database.Options.maxDebuffs
	end
	
	-- We create shortcuts to each of the small debuff frames
	for i = 1, maxDebuffs, 1 do
		self["debuff"..i] = _G[prefix.."Debuff"..i];
		self["debuffBorder"..i] = _G[prefix.."Debuff"..i.."Border"];
		self["debuffIcon"..i] = _G[prefix.."Debuff"..i.."Icon"];
		self["debuffCooldown"..i] = _G[prefix.."Debuff"..i.."Cooldown"];
		self["debuff"..i]["spellID"] = nil;
		self["debuff"..i]["expiration"] = nil;
		self["debuff"..i]["ID"] = _G[prefix.."Debuff"..i]:GetID();
	end
end



function ArenaLiveDebuffs_InitialisePlayer(self, unit)
	
	self.unit = unit;
	self:RegisterEvent("UNIT_AURA");
	self:SetScript("OnEvent", ArenaLiveDebuffs_OnEvent);
end



function ArenaLiveDebuffs_OnEvent(self, event, ...)
	
	local unit = ...;
	
	if ( event == "UNIT_AURA" ) and ( unit == self.unit) then
		ArenaLiveDebuffs_Refresh(self, unit)
	end
end



function ArenaLiveDebuffs_Refresh(self, unit)
	
	local maxDebuffs;
	local parent = self:GetParent();

	if (unit == "target" or self.realUnit == "targettarget") then 
		maxDebuffs = 10 
	else 
		maxDebuffs = ArenaLive_Database.Options.maxDebuffs 
	end
	
	for i = 1, maxDebuffs, 1 do
			
		local name, _, icon, _, dispelType, duration, expirationTime, _, _, _, spellID = UnitDebuff(unit, i);
			
			
		if (name) then
			
			self["debuffIcon"..i]:SetTexture(icon);
			
			local debuffColor = DebuffTypeColor[dispelType] or DebuffTypeColor["none"];
			self["debuffBorder"..i]:SetVertexColor(debuffColor.r, debuffColor.g, debuffColor.b);
			
			self["debuffCooldown"..i]:SetCooldown(expirationTime - duration, duration); -- Note for myself: expirationtime - complete duration gives us the starttime of the debuff. This way the debuff will alwyays be shown with the right cooldownprogress.
			self["debuff"..i]["spellID"] = spellID;
			self["debuff"..i]["expiration"] = expirationTime;
			self["debuff"..i]:Show();
			
		else
			self["debuff"..i]:Hide();
		end
		
	end
end





--[[ PORTRAIT OVERLAY FUNCTIONS ]]--
function ArenaLiveCcOverlay_OnLoad(self)
	
	local prefix = self:GetName();
	
	self.texture = _G[prefix.."Texture"];
	self.cooldown = _G[prefix.."Cooldown"];
	self.highPrioDebuff = {};
	self.highPrioDebuff.duration = 0;
	self.highPrioDebuff.priority = 0;
	self.highPrioDebuff.spellID = nil;
	self.highPrioDebuff.expirationTime = nil;
	self.highPrioDebuff.auraID = nil;
	self.highPrioDebuff.auraType = nil;
	
end



function ArenaLiveCcOverlay_InitialisePlayer(self, unit)

	self.unit = unit;
	
	-- Register for Event
	self:RegisterEvent("UNIT_AURA")
	
	self:SetScript("OnUpdate", ArenaLiveCcOverlay_OnUpdate);
	self:SetScript("OnEvent", ArenaLiveCcOverlay_OnEvent);
	
end



function ArenaLiveCcOverlay_OnEvent(self, event, ...)
	
	local unit = select(1, ...);
	
	if (unit == self.unit) and (event == "UNIT_AURA") then
		
		local parent = self:GetParent();		
		
		for i = 1, 40 do
		
			local name, _, icon, _, dispelType, duration, expirationTime, _, _, _, spellID = UnitBuff(unit, i)
			if (name) then
				ArenaLiveCcOverlay_UpdateCcOverlay(parent, spellID, duration, expirationTime, i, "Buff")
			else
				-- There are no auras left, so we can stop here
				break;
			end
			
		end

		for i = 1, 40 do
		
			local name, _, icon, _, dispelType, duration, expirationTime, _, _, _, spellID = UnitDebuff(unit, i)
			if (name) then
				ArenaLiveCcOverlay_UpdateCcOverlay(parent, spellID, duration, expirationTime, i, "Debuff")
			else
				-- There are no auras left, so we can stop here
				break;
			end
			
		end
		
	end
	
end



function ArenaLiveCcOverlay_UpdateCcOverlay(parent, spellID, duration, expirationTime, auraID, auraType)
	
	local spellPriority = nil;
		
	if (spellID == 76577 ) then
		duration = 5;
	end
	
	local startTime = expirationTime - duration;
	
	
	-- First we need to check, if we can find a match for our Debuff in our CC-Spell-List
	if (ArenaLive_SpellsToTrack.PortraitOverlay[spellID]) then
		
		spellPriority = ArenaLive_SpellsToTrack.PortraitOverlay[spellID];
		
			-- If expirationTime + SpellID + auraID are the same we don't need to update, since it is the current spell.
			if ((expirationTime ~= parent.ccOverlay.highPrioDebuff.expirationTime) and (spellID ~= parent.ccOverlay.highPrioDebuff.spellID)) or (auraID ~= parent.ccOverlay.highPrioDebuff.auraID ) then
				
				-- If we've found something new, we must compare it now to the current stored Debuff. If it is more important, then we can swap the values
				if (spellPriority > parent.ccOverlay.highPrioDebuff.priority) or ( spellPriority == parent.ccOverlay.highPrioDebuff.priority and duration > parent.ccOverlay.highPrioDebuff.duration ) then
					
					local name, _, icon = GetSpellInfo(spellID)
					
					parent.ccOverlay.highPrioDebuff.spellID = spellID;
					parent.ccOverlay.highPrioDebuff.priority = spellPriority;
					parent.ccOverlay.highPrioDebuff.duration = duration;
					parent.ccOverlay.highPrioDebuff.expirationTime = expirationTime;
					parent.ccOverlay.highPrioDebuff.auraID = auraID;
					parent.ccOverlay.highPrioDebuff.auraType = auraType;
					
					parent.ccOverlay.texture:SetTexture(icon);
					parent.ccOverlay.cooldown:SetCooldown(startTime, duration);
					parent.ccOverlay:Show();
				end
			end
	end
end



function ArenaLiveCcOverlay_OnUpdate(self, elapsed)

	if ( self.highPrioDebuff.spellID ) then
		
		self.highPrioDebuff.duration = self.highPrioDebuff.duration - elapsed;
		
			if ( self.highPrioDebuff.auraType == "Buff" ) then
				name, _, icon, _, dispelType, _, expirationTime, _, _, _, spellID = UnitBuff(self.unit, self.highPrioDebuff.auraID)
			else
				name, _, icon, _, dispelType, _, expirationTime, _, _, _, spellID = UnitDebuff(self.unit, self.highPrioDebuff.auraID)
			end
		
			-- If there is no debuff or the same auraID is now used by another spell, this means our CC isn't active anymore and must be resettet.
			if (not name) or (self.highPrioDebuff.spellID ~= spellID) then
				self.highPrioDebuff.spellID = nil;
				self.highPrioDebuff.auraID = nil;
				self.highPrioDebuff.auraType = nil;
				self.highPrioDebuff.duration = 0;
				self.highPrioDebuff.priority = 0;
				self:Hide();				
			end		
		
		if ( self.highPrioDebuff.duration <= 0 ) then
			
			-- This is the case, if there aren't any CCs left on the player
			self.highPrioDebuff.spellID = nil;
			self.highPrioDebuff.auraID = nil;
			self.highPrioDebuff.auraType = nil;
			self.highPrioDebuff.duration = 0;
			self.highPrioDebuff.priority = 0;
			self:Hide();
			
		end
	end
end

--[[ RACIAL AND TRINKET CD FUNCTIONS ]]--
function ArenaLiveCooldown_OnLoad(self)
	
	local prefix = self:GetName();
	self.texture = _G[prefix.."Texture"];
	self.cooldown = _G[prefix.."Cooldown"];	
	
end



function ArenaLiveCooldown_InitialisePlayer(self, unit, race, class, cdType)

	self.unit = unit;
	local faction = UnitFactionGroup(unit);

	if (race == "Draenei") or (race == "Orc") or (race =="BloodElf") then
		
		self.sharedCD = ArenaLive_SpellsToTrack.Racials[race][class][3];
	else
		
		self.sharedCD = ArenaLive_SpellsToTrack.Racials[race][3];
	end
	
	if (cdType == "trinket" ) then
		if ( faction == "Alliance" ) then
			self.texture:SetTexture("Interface\\ICONS\\INV_Jewelry_TrinketPVP_01");
		else
			self.texture:SetTexture("Interface\\ICONS\\INV_Jewelry_TrinketPVP_02");
		end
	
		self.spellID =  ArenaLive_SpellsToTrack.Trinket
	else
		if (race == "Draenei") or (race == "Orc") or (race =="BloodElf") then
		
			self.spellID =  ArenaLive_SpellsToTrack.Racials[race][class][1]
			self.cooldownTime = ArenaLive_SpellsToTrack.Racials[race][class][2];
		else
		
			self.spellID =  ArenaLive_SpellsToTrack.Racials[race][1]
			self.cooldownTime = ArenaLive_SpellsToTrack.Racials[race][2];
		end		
		
		local _, _, icon = GetSpellInfo(self.spellID);
		
		if (icon) then
			self.texture:SetTexture(icon);
		end
		
	end
	
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED");
	self:SetScript("OnEvent", ArenaLiveCooldown_OnEvent);
end



function ArenaLiveCooldown_OnEvent(self, event, ...)
	
	local unit, _, _, _, spellID = ...;
	
	if ( event == "UNIT_SPELLCAST_SUCCEEDED" ) and ( unit == self.unit ) and (spellID == self.spellID) then
		
		if ( spellID == ArenaLive_SpellsToTrack.Trinket ) then
			
			self.cooldown:SetCooldown(GetTime(), 120);
			
			if (self.sharedCD > 0 ) then
				local frameParent = self:GetParent();
				frameParent.racial.cooldown:SetCooldown(GetTime(), self.sharedCD);
			end
		else
			self.cooldown:SetCooldown(GetTime(), self.cooldownTime);
			if (self.sharedCD > 0 ) then
				local frameParent = self:GetParent();
				frameParent.trinket.cooldown:SetCooldown(GetTime(), self.sharedCD);
			end			
		end
	end
end





--[[ COOLDOWNTRACKER FUNCTIONS ]]--
function ArenaLiveCooldownTracker_OnLoad(self, side)
	
	local prefix = self:GetName();
	self.side = side;
	self.cooldownList = {};
	self.unitIDs = {};
	
	-- Register all our Cooldown-Icons
	for i = 1, 30 do
	
		self["cd"..i] = _G[prefix.."CD"..i];
		self["cd"..i]["id"] = self["cd"..i]:GetID();
		self["cd"..i]["texture"] = _G[prefix.."CD"..i.."Texture"];
		self["cd"..i]["cooldown"] = _G[prefix.."CD"..i.."Cooldown"];
		self["cd"..i]["unit"] = nil;
		self["cd"..i]["spellID"] = nil;
		self["cd"..i]["enabled"] = nil;
	end
	
	-- Register for the needed Event
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED");
	
end



function ArenaLiveCooldownTracker_OnEvent(self, event, ...)
	
	if ( event == "UNIT_SPELLCAST_SUCCEEDED" ) then
		
		local spellID = select(5, ...);

		if (ArenaLive_SpellsToTrack["Cooldowns"][spellID]) then
			ArenaLive_SystemMessage("Cooldowntracker - SpellID found!", true)
			
			local unit = select(1, ...);
			local match = false;	
			
			for key, value in pairs(self.unitIDs) do
				if (unit == value) then
					ArenaLive_SystemMessage("Match found!", true);
					match = true;
				end
			end	
			
			if (match) then
				ArenaLiveCooldownTracker_AddCooldown(self, unit, spellID)
			end
		end
		
	end
end



function ArenaLiveCooldownTracker_AddCooldown(self, unit, spellID)

	-- Fill our Cooldown-Table with infos.
	local duration = ArenaLive_SpellsToTrack["Cooldowns"][spellID];
	local valuesToInsert = { ["spellID"] = spellID, ["unit"] = unit, ["startTime"] = GetTime(), ["duration"] = duration;}
	table.insert(self.cooldownList, valuesToInsert);
	
	-- Now add our data into the icon
	local _, _, icon = GetSpellInfo(spellID);	
	local n = #(self.cooldownList);
	
	self["cd"..n]["texture"]:SetTexture(icon);
	self["cd"..n]["cooldown"]:SetCooldown(GetTime(), duration);	
	self["cd"..n]["unit"] = unit; -- Is needed to checck if a cooldown needs to be updated, as soon as another cooldown is finished
	self["cd"..n]["spellID"] = spellID; -- Is needed to checck if a cooldown needs to be updated, as soon as another cooldown is finished
	self["cd"..n]["enabled"] = true;
	self["cd"..n]:SetScript("OnUpdate", ArenaLiveCooldownTrackerCD_OnUpdate);
	self["cd"..n]:Show();
	
end



function ArenaLiveCooldownTracker_UpdateCooldowns(cooldownTracker, id)

	table.remove(cooldownTracker.cooldownList, id);
	local n = #(cooldownTracker.cooldownList);
	
	ArenaLive_SystemMessage("ArenaLiveCooldownTracker_UpdateCooldowns - n = "..tostring(n), true);
	-- If there was only 1 Cooldown on the list and this one ran out, we don't need to update
	if ( n > 0 ) then
		for i = 1, 30 do
			
			if (i <= n ) then
				ArenaLive_SystemMessage("i: "..i..", SpellID: "..tostring(cooldownTracker.cooldownList[i]["spellID"])..", duration: "..cooldownTracker.cooldownList[i]["duration"], true);
				-- Prevent refreshing Cooldownanimations that don't need to be refreshed, because this always looks kind of strange.
				if ( cooldownTracker["cd"..i]["spellID"] ~= cooldownTracker.cooldownList[i]["spellID"] ) or ( cooldownTracker["cd"..i]["unit"] ~= cooldownTracker.cooldownList[i]["unit"]) then
					local spellID = cooldownTracker.cooldownList[i]["spellID"];
					local _, _, icon = GetSpellInfo(spellID);
					cooldownTracker["cd"..i]["texture"]:SetTexture(icon);
					cooldownTracker["cd"..i]["cooldown"]:SetCooldown(cooldownTracker.cooldownList[i]["startTime"], ArenaLive_SpellsToTrack["Cooldowns"][spellID]);	
					cooldownTracker["cd"..i]["spellID"] = spellID;
					cooldownTracker["cd"..i]["unit"] = cooldownTracker.cooldownList[i]["unit"];
					cooldownTracker["cd"..i]:SetScript("OnUpdate", ArenaLiveCooldownTrackerCD_OnUpdate);
					cooldownTracker["cd"..i]["enabled"] = true;
					cooldownTracker["cd"..i]:Show();
				end
			else
				-- Reset Icons, that we don't need anymore.
				if ( cooldownTracker["cd"..i]["enabled"] ) then
					cooldownTracker["cd"..i]["enabled"] = nil;
					cooldownTracker["cd"..i]["unit"] = nil;
					cooldownTracker["cd"..i]["spellID"] = nil;
					cooldownTracker["cd"..i]:SetScript("OnUpdate", nil);
					cooldownTracker["cd"..i]:Hide();
				else
					-- We can break the for-loop here, since there are no active frames left.
					break;
				end
			end
		end
	end
end



function ArenaLiveCooldownTracker_UpdateUnitID(self, unit, oldUnit, mode)
	
	local gameModeNumber = ArenaLive_GetGameModeNumber();
	
	if ( mode == "add" ) then
		local match = nil;
		
		for key, value in pairs(self.unitIDs) do
		
			if ( value == unit ) then
				match = true;
			end
			
		end
		
		if ( match ) then
			ArenaLive_SystemMessage("UnitID already has a cooldown-tracker entry, sticking with old one.", true);
		else
			ArenaLive_SystemMessage("Creating Cooldowntracker entry for unit: "..tostring(unit), true);
			table.insert(self.unitIDs, unit);
		end
	
	elseif ( mode == "modify" ) then
		
		local entryToDelete;
		
		for key, value in pairs(self.unitIDs) do
			if ( value == oldUnit ) then		
				entryToDelete = key;		
			end
		end	
		ArenaLive_SystemMessage("Modified Cooldowntracker for entry \""..oldUnit.."\" and set new unit: "..tostring(unit), true);
		table.remove(self.unitIDs, entryToDelete);
		table.insert(self.unitIDs, unit);
		
	elseif ( mode == "delete" ) then
	
		local entryToDelete;
	
		for key, value in pairs(self.unitIDs) do
			if ( value == unit ) then		
				entryToDelete = key;		
			end
		end
		
		if (entryToDelete) then
			ArenaLive_SystemMessage("Deleting cooldowntracker entry for: "..tostring(unit), true);
			table.remove(self.unitIDs, entryToDelete);
		else
			ArenaLive_SystemMessage("Can't find the unit in the cooldowntracker-unitlist, nothing deleted.", true);
		end
	end
end



function ArenaLiveCooldownTrackerCD_OnUpdate(self, elapsed)
	
	if (self.enabled) then
		
		local parent = self:GetParent();
		
		if (parent.cooldownList[self.id]["duration"] <= 0) then
			
			self.enabled = nil;
			self.unit = nil;
			self.spellID = nil;
			self:SetScript("OnUpdate", nil);
			self:Hide();
			
			
			ArenaLiveCooldownTracker_UpdateCooldowns(parent, self.id)
		else
			parent.cooldownList[self.id]["duration"] = parent.cooldownList[self.id]["duration"] - elapsed;
		end
		
	end
end










--[[ SCOREBOARD FUNCTIONS ]] --

function ArenaLiveScoreboard_OnLoad(self)

	local prefix = self:GetName();
	
	self.leftTeam = _G[prefix.."TeamLeftName"];
	self.leftTeamText = _G[prefix.."TeamLeftNameText"];
	
	self.rightTeam = _G[prefix.."TeamRightName"];
	self.rightTeamText = _G[prefix.."TeamRightNameText"];
	
	self.scoreText = _G[prefix.."ScoreAndTimeScore"];
	self.timeText = _G[prefix.."ScoreAndTimeTheTime"];
	
	self.timerActive = false;
	self.elapsed = 0;
	self.sec = 0;
	self.min = 0;
	
	self.leftTeam:RegisterForClicks("LeftButtonUp", "RightButtonUp");
	self.rightTeam:RegisterForClicks("LeftButtonUp", "RightButtonUp");
end



function ArenaLiveScoreboard_SetTeamName(frame, side)

	if (side == "left" ) then
	
		frame.leftTeamText:SetText(ArenaLive_Database.TeamLeft.Name)
	
	else
	
		frame.rightTeamText:SetText(ArenaLive_Database.TeamRight.Name)
	
	end

end



function ArenaLiveScoreboard_UpdateScore(self, button, down, side)

	local scoreboard = self:GetParent();

	if ( side == "left" ) then
		
		if ( button == "LeftButton" ) then
			
			ArenaLive_Database.TeamLeft.Score = ArenaLive_Database.TeamLeft.Score + 1;
			
		elseif ( button == "RightButton" ) then

			ArenaLive_Database.TeamLeft.Score = ArenaLive_Database.TeamLeft.Score - 1;
			
			-- We need to prevent people from getting into negative numbers, so we will reset the stats to zero, if we're getting into negatives
			if ( ArenaLive_Database.TeamLeft.Score < 0 ) then
				
				ArenaLive_Database.TeamLeft.Score = 0;
				
			end
			
		end
		
	else
	
		if ( button == "LeftButton" ) then
			
			ArenaLive_Database.TeamRight.Score = ArenaLive_Database.TeamRight.Score + 1;
			
		elseif ( button == "RightButton" ) then

			ArenaLive_Database.TeamRight.Score = ArenaLive_Database.TeamRight.Score - 1;
			
			-- We need to prevent people from getting into negative numbers, so we will reset the stats to zero, if we're getting into negatives
			if ( ArenaLive_Database.TeamRight.Score < 0 ) then
				
				ArenaLive_Database.TeamRight.Score = 0;
				
			end
			
		end	
	
	end
	
	ArenaLiveScoreboard_SetScoreText()
end



function ArenaLiveScoreboard_SetScoreText()

	local scoreboard = _G["ArenaLive_Scoreboard"];
	
	local score = tostring(ArenaLive_Database.TeamLeft.Score).." : "..tostring(ArenaLive_Database.TeamRight.Score)
	
	scoreboard.scoreText:SetText(score);
	
end



function ArenaLiveScoreboard_ResetTeamScores()
	
	ArenaLive_Database.TeamLeft.Score  = 0;
	ArenaLive_Database.TeamRight.Score = 0;
	
	ArenaLiveScoreboard_SetScoreText()
end



function ArenaLiveScoreboard_StartTimer()

	local scoreboard = _G["ArenaLive_Scoreboard"];
	
	scoreboard.timerActive = true;
	scoreboard:SetScript("OnUpdate", ArenaLiveScoreboard_OnUpdate);
end



function ArenaLiveScoreboard_OnUpdate(self, elapsed)

	if ( self.timerActive ) then
		
		self.elapsed = self.elapsed + elapsed
		
		if ( self.elapsed >= 1 ) then
			
			self.sec = self.sec + 1;
			self.elapsed = 0;
			
			-- One minutes has passed, so we add +1 to minutes
			if ( self.sec >= 60 ) then
				self.sec = 0;
				self.min = self.min + 1;
			end
			
		end
		
		local secString, minString = nil;
		
		if ( self.sec < 10 ) then
			secString = "0"..tostring(self.sec);
		else
			secString = tostring(self.sec);
		end

		if ( self.min < 10 ) then
			minString = "0"..tostring(self.min);
		else
			minString = tostring(self.min);
		end
		
		self.timeText:SetText(minString..":"..secString);
		
		-- We've reached our Timelimit, so we reset the timer
		if ( self.min == ArenaLive_Database.Options.timeLimitMin ) and ( self.sec == ArenaLive_Database.Options.timeLimitSec ) then
			
			self.timerActive = false;
			SendChatMessage("*** TIME'S UP! ***", "BATTLEGROUND");
			
		end
		
	else
	
		ArenaLiveScoarboard_ResetTimer()
		
	end
end

function ArenaLiveScoarboard_ResetTimer()

	local scoreboard = _G["ArenaLive_Scoreboard"];

	scoreboard:SetScript("OnUpdate", nil);
	scoreboard.sec = 0;
	scoreboard.min = 0;
	scoreboard.elapsed = 0;
	scoreboard.timeText:SetText("00:00");
	
end